/**
 * @file basemotor.cpp
 * @author Keith Holman
 * @date Created: Sunday, November 30, 2008
 *
 * @brief Defines the base atomic model for the motor
 */

#include "basemotor.h"
#include "mainsimu.h"   // MainSimulator::Instance().getParameter( ... )
#include "NXT++.h"

BaseMotor::BaseMotor( const std::string& name )
: Atomic( name ),
  _motorPower( 20.0 ),
  _port( 0 )
{
  _motorPower =
    atof( MainSimulator::Instance().getParameter(
      description(), "motorPower" ).c_str() );
  std::string port_param = MainSimulator::Instance().getParameter(
          description(), "port" );
  switch ( port_param.at( 0 ) )
  {
  case 'A':
  case 'a':
    _port = OUT_A;
    break;
  case 'B':
  case 'b':
    _port = OUT_B;
    break;
  case 'C':
  case 'c':
    _port = OUT_C;
    break;
  default:
    _port = atoi( port_param.c_str() );
    if ( _port < OUT_A ) _port = OUT_A;
    else if ( _port > OUT_C ) _port = OUT_C;
  }
}

BaseMotor::~BaseMotor()
{
  // Stop the motor
  NXT::Motor::Stop( _port, true );
}
