#ifndef BASE_MOTOR_H
#define BASE_MOTOR_H
/**
 * @file basemotor.h
 * @author Keith Holman
 * @author Jeremy Kuzub
 * @date Created: Sunday, November 30, 2008
 *
 * @brief Defines the base atomic model for a motor
 */

#include "atomic.h"     // class Atomic

/**
 * @author Keith Holman
 * @date Created: Sunday, November 30, 2008
 *
 * @brief Atomic model for a motor
 */
class BaseMotor : public Atomic
{
public:
  /**
   * @brief Constructor
   * @param name      name of the atomic model
   */
  BaseMotor( const std::string& name );

  /**
   * @brief Destructor
   */
  virtual ~BaseMotor();

protected:
  /** @brief Amont of power to use when directing the motor to move */
  Value _motorPower;
  /** @brief Port the motor is connected to within NXT */
  int _port;

private:
  /**
   * @brief Copy constructor
   * @param obj       object to copy
   */
  BaseMotor( const BaseMotor &obj );
  /**
   * @brief Assignment operator
   * @param obj       object to copy
   * @return a reference to @c this object
   */
  BaseMotor& operator=( const BaseMotor &obj );
};

#endif   // ifndef BASE_MOTOR_H
