/**
 * @file engager.cpp
 * @author Keith Holman
 * @date Created: Tuesday, October 14, 2008
 * @date Modified: Sunday, November 30, 2008
 *
 * @brief Defines the atomic model for the chalk engager
 */

#include "engager.h"
#include "message.h"    // class ExternalMessage, InternalMessage
#include "mainsimu.h"   // MainSimulator::Instance().getParameter( ... )
#include "NXT++.h"      // For controlling the NXT motor
#include <cstdlib>

Engager::Engager( const std::string& name )
: BaseMotor( name ),
  _in( addInputPort( "in" ) ),
  _out( addOutputPort( "out" ) ),
  _chalkEngaged( false ),
  _preparationTime( 0, 0, 10, 0 )
{
  _chalkEngaged = atoi(
  MainSimulator::Instance().getParameter(
    description(), "engaged" ).c_str() );
  std::string time(
  MainSimulator::Instance().getParameter(
    description(), "preparation" ) ) ;

  if( time != "" )
    _preparationTime = time;
}

Model &Engager::initFunction()
{
  this->passivate();
  return *this;
}

Model &Engager::externalFunction( const ExternalMessage &msg )
{
  if( msg.port() == _in )
  {
    bool new_value = static_cast < bool >( msg.value() );
    if ( _chalkEngaged != new_value )
    {
      _chalkEngaged = new_value;
      NXT::Motor::ResetRotationCount( _port, false );
      if ( _chalkEngaged )
        NXT::Motor::SetForward( _port, _motorPower );
      else
        NXT::Motor::SetReverse( _port, _motorPower );
      holdIn( active, _preparationTime );
    }
  }
  return *this;
}

Model &Engager::internalFunction( const InternalMessage & )
{
  passivate();
  return *this;
}

Model &Engager::outputFunction( const InternalMessage &msg )
{
  NXT::Motor::Stop( _port, true );
  sendOutput( msg.time(), _out, _chalkEngaged );
  return *this ;
}

std::string Engager::className() const
{
  return "Engager";
}
