#ifndef ENGAGER_H
#define ENGAGER_H
/**
 * @file engager.h
 * @author Keith Holman
 * @date Created: Tuesday, October 14, 2008
 * @date Modified: Sunday, November 30, 2008
 *
 * @brief Defines the atomic model for the chalk engager
 */

#include "basemotor.h"     // class Atomic

/**
 * @author Keith Holman
 * @date Created: Tuesday, October 14, 2008
 * @date Modified: Sunday, November 30, 2008
 *
 * @brief Atomic model for the chalk engager
 */
class Engager : public BaseMotor
{
public:
  /**
   * @brief Constructor
   * @param name      name of the atomic model
   */
  Engager( const std::string& name = "Engager" );
  /**
   * @brief returns the name of the class of this atomic model
   * @return the name of the class of this atomic model
   */
  virtual std::string className() const;

protected:
  /**
   * @brief Initialization function
   * @return a reference to @c this object
   */
	Model &initFunction();
  /**
   * @brief External transition function
   * @param msg   transition message object
   * @return a reference to @c this object
   */
	Model &externalFunction( const ExternalMessage &msg );
  /**
   * @brief Internal transition function
   * @param msg   transition message object
   * @return a reference to @c this object
   */
	Model &internalFunction( const InternalMessage &msg );
  /**
   * @brief Output function
   * @param msg   transition message object
   * @return a reference to @c this object
   */
	Model &outputFunction( const InternalMessage &msg );

private:
  /**
   * @brief Copy constructor
   * @param obj       object to copy
   */
  Engager( const Engager &obj );
  /**
   * @brief Assignment operator
   * @param obj       object to copy
   * @return a reference to @c this object
   */
  Engager& operator=( const Engager &obj );

  // Ports
  /** @brief Input port */
  const Port &_in;
  /** @brief Output port */
  Port &_out;
  /** @brief Internally stored value */
  bool _chalkEngaged;

  /**
   * @brief time required to complete action
   */
  Time _preparationTime;
};

#endif   // ifndef ENGAGER_H
