#ifndef MOTOR_H
#define MOTOR_H
/**
 * @file motor.h
 * @author Keith Holman
 * @author Jeremy Kuzub
 * @date Created: Tuesday, October 14, 2008
 * @date Modified: Sunday, November 30, 2008
 *
 * @brief Defines the atomic model for the motor
 */

#include "basemotor.h"     // class Atomic

/**
 * @author Keith Holman
 * @author Jeremy Kuzub
 * @date Created: Tuesday, October 14, 2008
 * @date Created: Sunday, November 30, 2008
 *
 * @brief Atomic model for the motor
 */
class Motor : public BaseMotor
{
public:
  /**
   * @brief Constructor
   * @param name      name of the atomic model
   */
  Motor( const std::string& name = "Motor" );

  /**
   * @brief Destructor
   */
  virtual ~Motor();

  /**
   * @brief returns the name of the class of this atomic model
   * @return the name of the class of this atomic model
   */
  virtual std::string className() const;

protected:
  /**
   * @brief Initialization function
   * @return a reference to @c this object
   */
	Model &initFunction();
  /**
   * @brief External transition function
   * @param msg   transition message object
   * @return a reference to @c this object
   */
	Model &externalFunction( const ExternalMessage &msg );
  /**
   * @brief Internal transition function
   * @param msg   transition message object
   * @return a reference to @c this object
   */
	Model &internalFunction( const InternalMessage &msg );
  /**
   * @brief Output function
   * @param msg   transition message object
   * @return a reference to @c this object
   */
	Model &outputFunction( const InternalMessage &msg );

private:
  /**
   * @brief Copy constructor
   * @param obj       object to copy
   */
  Motor( const Motor &obj );
  /**
   * @brief Assignment operator
   * @param obj       object to copy
   * @return a reference to @c this object
   */
  Motor& operator=( const Motor &obj );

  // Ports
  /** @brief Input port */
  const Port &_in;
  /** @brief Output port */
	Port &_out;

  /** @brief Circumference of the motor */
  Value _circumference;
  /** @brief Amount of cable currently expelled */
  Value _expelled;
  /** @brief Target amount of cable to be expelled at end of transition */
  Value _target;
  /** @brief Total amount of cable on the motor */
  Value _total;
  /** @brief Whether movement caused an error */
  bool _success;

  /** @brief Number of rotations to perform per second */
  Value _rotationsPerSecond;
};

#endif   // ifndef MOTOR_H
