/**
 * @file point.cpp
 * @author Keith Holman
 * @date Created: Tuesday, October 14, 2008
 *
 * @brief Defines the Point object type
 */

#include "point.h"
#include "compare.h"
#include <string.h>

Point::Point()
: _x( 0 ), _y( 0 )
{
}

Point::Point( Point::coord_t x, Point::coord_t y )
: _x( x ), _y( y )
{
}

bool Point::operator==( const Point& p ) const
{
  return is_equal( _x, p._x ) && is_equal( _y, p._y );
}


Point& Point::operator+=( const Point& p )
{
  _x += p._x;
  _y += p._y;
  return *this;
}

Point& Point::operator-=( const Point& p )
{
  _x -= p._x;
  _y -= p._y;
  return *this;
}


Point::Point( const Value &v )
: _x( 0 ), _y( 0 )
{
  const char* p = reinterpret_cast<const char*>(&v);
  memcpy( &_x, p, sizeof( _x ) );
  memcpy( &_y, p + (sizeof(Value)/2), sizeof( _y ) );
}

Value Point::toValue() const
{
  Value result;
  char* p = reinterpret_cast<char*>(&result);
  memcpy( p, &_x, sizeof( _x ) );
  memcpy( p + (sizeof(Value)/2), &_y, sizeof( _y ) );
  return result;
}
