#ifndef QUEUE_H
#define QUEUE_H
/**
 * @file queue.h
 * @author Keith Holman
 * @date Created: Sunday, October 19, 2008
 *
 * @brief Atomic model for the queue
 */

#include "atomic.h"     // class Atomic
#include <queue>

/**
 * @author Keith Holman
 * @date Created: Tuesday, October 14, 2008
 *
 * @brief Atomic model for the queue
 */
class MyQueue : public Atomic
{
public:
  /**
   * @brief Constructor
   * @param name      name of the atomic model
   */
  MyQueue( const std::string& name = "MyQueue" );
  /**
   * @brief returns the name of the class of this atomic model
   * @return the name of the class of this atomic model
   */
  virtual std::string className() const;

protected:
  /**
   * @brief Initialization function
   * @return a reference to @c this object
   */
	Model &initFunction();
  /**
   * @brief External transition function
   * @param msg   transition message object
   * @return a reference to @c this object
   */
	Model &externalFunction( const ExternalMessage &msg );
  /**
   * @brief Internal transition function
   * @param msg   transition message object
   * @return a reference to @c this object
   */
	Model &internalFunction( const InternalMessage &msg );
  /**
   * @brief Output function
   * @param msg   transition message object
   * @return a reference to @c this object
   */
	Model &outputFunction( const InternalMessage &msg );

private:
  /**
   * @brief Copy constructor
   * @param obj       object to copy
   */
  MyQueue( const MyQueue &obj );
  /**
   * @brief Assignment operator
   * @param obj       object to copy
   * @return a reference to @c this object
   */
  MyQueue& operator=( const MyQueue &obj );

  // Ports
  /** @brief Input port */
  const Port &_in;
  /** @brief Input port */
  const Port &_ready;
  /** @brief Output port */
  Port &_out;
  /**
   * @brief Whether the queue is waiting for a response back before sending
   * the next item
   */
  bool _waiting;

  /** @brief Internal queue for storing inputs */
  std::queue< Value >  _queue;
};

#endif   // ifndef QUEUE_H
