/*******************************************************************
*
*  DESCRIPTION: class AtomicData
*
*  AUTHOR: Ezequiel Glinsky
*
*  EMAIL: mailto://eglinsky@dc.uba.ar
*
*  DATE: 11/2001
*
*******************************************************************/

#ifndef __ATOMDATA_H
#define __ATOMDATA_H

/** include files **/
//#include "fstream.h"
//#include "pProcessorState.h"  //ProcessorState
#include "modelid.h"    // class ModelId
#include "cdTime.h"    // class Time
#include "except.h"
#include "value.h"
//#include "msgbag.h"
#include "message.h"
#include "atomic.h"      // class Atomic

/** forward declarations **/
class Model ;

//class BasicMsgValue;
class Message;
class InitMessage ;
class InternalMessage ;
//class BasicOutputMessage;
class OutputMessage ;
//class BasicExternalMessage;
class ExternalMessage ;
class DoneMessage ;
//class CollectMessage;
//class MessageBag;
//class OutputSyncMessage;
class Atomic;

class Message;

class SingleProcessorAdmin ;
class Port ;

/** declarations **/
typedef ModelId ProcId ;


class AtomicData
{
public:
  virtual ~AtomicData();   // Destructor

  // the Processor Id
  const ProcId &id() const
  { return ident;}

  // return the asociate model
  Model &model() ;
  const Model &model() const ;

  // return a pointer to the asociate state
  //AtomicState* modelState();
  //const AtomicState* modelState() const;

  const Time &nextChange() const ;
  const Time &lastChange() const ;
  const Time absoluteNext() const {
    return nextChange() + lastChange();
  }

  const std::string asString() const ;

  virtual const std::string description() const;

  static const ProcId InvalidId ;

  AtomicData &debug();

  AtomicData() ;        // Default constructor
  AtomicData( Atomic * atomic) ;
  AtomicData( const AtomicData & );  //Copy constructor
  AtomicData &operator =( const AtomicData & ) ;    // Assignment operator
  int operator ==( const AtomicData & ) const ;    // Equality operator

protected:
  friend class Model ;
  friend class SingleProcessorAdmin ;
  friend class Root ;
  friend class MainSimulator ;
  friend class FlatDEVSCoordinator ;
  friend class Atomic ;
  friend class Processor ;

  AtomicData &id( const ProcId & ) ;

  AtomicData &nextChange( const Time & ) ;
  AtomicData &lastChange( const Time & ) ;

  Model *pmodel();

  unsigned createlog;

  int logIndex;
  int numOutFiles;

  //MessageBag externalMsgs;
  ExternalMessage externalMsgs;
  //VTime lastMsgTime;

private:

  Model *mdl ;
  //AtomicState* modelstate;
  ProcId ident ;

  Time next ;
  Time last ;

};  // class AtomicData

/** inline **/
inline
  AtomicData::AtomicData()
  : mdl( NULL )
{}

/*
inline
  AtomicState* AtomicData::modelState()
{
  return modelstate ;
}

inline
  const AtomicState* AtomicData::modelState() const
{
  return modelstate ;
}
*/

inline
  Model &AtomicData::model()
{
  return *mdl ;
}

inline
  const Model &AtomicData::model() const
{
  MASSERT( mdl ) ;
  return *mdl ;
}

inline
  const Time &AtomicData::nextChange() const
{
  return next;
}

inline
  const Time &AtomicData::lastChange() const
{
  return last;
}

inline
  AtomicData &AtomicData::nextChange( const Time &t )
{
  next = t ;
  return *this ;
}

inline
  AtomicData &AtomicData::lastChange( const Time &t )
{
  last = t ;
  return *this ;
}

inline
  const std::string AtomicData::asString() const
{
  return description() + "(" + id() + ")" ;
}


inline
  Model *AtomicData::pmodel()
{
  return mdl ;
}

#endif   //__ATOMDATA_H
