/*******************************************************************
*
*  DESCRIPTION: class Atomic
*
*  AUTHOR:    Amir Barylko & Jorge Beyoglonian 
*  Version 2: Daniel A. Rodriguez
*
*  EMAIL: mailto://amir@dc.uba.ar
*         mailto://jbeyoglo@dc.uba.ar
*         mailto://drodrigu@dc.uba.ar
*
*  DATE: 27/6/1998
*  DATE: 25/4/1999
*
*******************************************************************/

/** include files **/
#include "atomic.h"             // base header
#include "message.h"	//class InternalMessage ExternalMessage

/*******************************************************************
* Function Name: holdIn
* Description: Changes the atomic state and the time left to the next change 
********************************************************************/
Model &Atomic::holdIn( const State &s, const Time &t )
{
	state( s );
	nextChange( t );
	return *this;
}

/*******************************************************************
* Function Name: passivate
* Description: Sets thes passive state and the next change to infinity 
********************************************************************/
Model &Atomic::passivate()
{
	state( passive );
	nextChange( Time::Inf );
	return *this;
}

/*******************************************************************
* Function Name: confluentFunction
* Description: Executes the default behavior of the confluent function
********************************************************************/
Model &Atomic::confluentFunction ( const InternalMessage &intMsg, 
                                   const MessageBag &extMsgs )
{
        //Default behavior for confluent function:
        //Proceed with the internal transition and then with the external
        internalFunction( intMsg );

        //Set the elapsed time to 0
        lastChange( intMsg.time() );

        //Call the external function
        externalFunction( extMsgs );

        return *this;

}//confluentFunction

#if 0
/*******************************************************************
* Function Name: confluentFunction
* Description: Executes the default behavior of the confluent function
********************************************************************/
Model &Atomic::confluentFunction ( const InternalMessage &intMsg )
{
  confluentFunction( intMsg, *pendingExtMsg_pm );
  return *this;

}//confluentFunction
#endif

#if 0
/*******************************************************************
* Function Name: addPendingExtMsg
* Description: 
********************************************************************/
Model &Atomic::addPendingExtMsg(ExternalMessage *pendingExtMsg_p)
{
  if (pendingExtMsg_pm){
    InvalidModelIdException e ;
    e.addText( std::string( "pendingExtMsg_rm is not NULL" ));
    MTHROW( e ) ;
  }//if

  pendingExtMsg_pm = pendingExtMsg_p;

  return *this;
}
#endif

/*******************************************************************
* Function Name: externalFunction
* This function is provided for backward compatibility between DEVS
* and parallel DEVS code. Please, do not make abstract since it will
* require models to define this function which is obsolete for
* Parallel DEVS
********************************************************************/
Model &Atomic::externalFunction( const ExternalMessage & )
{
  MException e ;
  e.addText("Can not call Atomic::externalFunction( const ExternalMessage&)!");
  MTHROW( e ) ;
}

/*******************************************************************
* Function Name: externalFunction
* Description: The default definition allows to maintain backward compatibility
* it can only be used for external messages containing a real value!
********************************************************************/
Model &Atomic::externalFunction ( const MessageBag & msgs )
{
  MessageBag::iterator cursor = msgs.begin();

  for( ; cursor != msgs.end(); cursor++ )
  {
    externalFunction( *(( ExternalMessage* )( *cursor )) );
  }

  return *this;
}

Model &Atomic::StartFunction(const StartMessage & msg)
{
  std::cout<<"Atomic : StartFunction "<<std::endl; 
  return *this;
}
