/*******************************************************************
*
*  DESCRIPTION: class Atomic
*
*  AUTHOR:    Amir Barylko & Jorge Beyoglonian 
*  VERSION 2: Daniel Rodriguez
*
*  EMAIL: mailto://amir@dc.uba.ar
*         mailto://jbeyoglo@dc.uba.ar
*         mailto://drodrigu@dc.uba.ar
*
*  DATE: 27/6/1998
*  DATE: 25/4/1999 (v2)
*
*******************************************************************/

#ifndef __ATOMIC_H
#define __ATOMIC_H

/** include files **/
#include "model.h"   //class Model
#include "message.h" //class ExternalMessage
#include "msgbag.h" //class MessageBag

/** foward declarations **/
class MainSimulator ;
class Simulator ;
class ExternalMessage ;
class InternalMessage ;
class MessageBag;

#define  ATOMIC_NAME    "Atomic"

/** definitions **/
class Atomic : public Model
{
public:
  virtual ~Atomic()  // Destructor
  { 
    //if (pendingExtMsg_pm) delete pendingExtMsg_pm;
  }


  Model::ModelType mType() { return Model::atomicType; }

protected:
  friend class Simulator ;
  friend class FlatDEVSCoordinator ;

  enum State
  {
    active,
    passive
  } ;

  Atomic( const std::string &name = "Atomic" )  // Constructor
      //: Model( name ), st( passive ), pendingExtMsg_pm(NULL)
      : Model( name ), st( passive )
      {}
  
  Atomic( const Atomic & ) ;     // Copy constructor

  Atomic &operator = ( const Atomic & );  // Assignment operator

  int operator ==( const Atomic & ) const; // Equality operator

  virtual Model &initFunction() = 0 ;

  virtual Model &externalFunction( const ExternalMessage & );
  virtual Model &externalFunction( const MessageBag & );

  virtual Model &internalFunction( const InternalMessage & ) = 0 ;

  virtual Model &outputFunction( const InternalMessage & ) = 0 ;
  virtual Model &StartFunction(const StartMessage &);
  //virtual Model &confluentFunction ( const InternalMessage & );
  virtual Model &confluentFunction ( const InternalMessage &, const MessageBag & );
  Model &holdIn( const State &, const Time & ) ;

  Model &passivate();

  Model &state( const State &s )
    { st = s; return *this; }

  const State &state() const
    {return st;}
  
  virtual std::string className() const
    {return ATOMIC_NAME;}

  //Model &addPendingExtMsg(ExternalMessage *);

  //int isPedingExtMsg() { return (pendingExtMsg_pm == NULL)?  0 : 1;}
private:
  friend class MainSimulator ;
  State st;
  //ExternalMessage* pendingExtMsg_pm;

};  // class Atomic

#endif   //__ATOMIC_H

