/*******************************************************************
*
*  DESCRIPTION: class Time
*
*  AUTHOR: Amir Barylko & Jorge Beyoglonian 
*
*  EMAIL: mailto://amir@dc.uba.ar
*         mailto://jbeyoglo@dc.uba.ar
*
*  DATE: 27/6/1998
*  Edited By: Mohammad Moallemi
* 
*  Date: 22/10/2008
* 
*  Email: moallemi@sce.carleton.ca
*  currentTime() bug fixed
*******************************************************************/

/** include files **/
#include "cdTime.h"        // base class
#include "stringp.h"     // operator + int
#include <time.h>	 // clock()		
#include <sys/time.h>    //gettimeofday()

/** public data **/
const Time Time::Zero ;    // Zero constant
const Time Time::Inf( 32767, 59, 59, 999 ) ;  // Infinity constant
  //Mohammad bug fix
   timeval starttv, curtv;
  int dummy = gettimeofday(&starttv,NULL);//get start time
  //end Mohammad

/*******************************************************************
* Function Name: Time
* Description: copy constructor
********************************************************************/
Time::Time( const Time &t )
: hour( t.hours() )
, min( t.minutes() )
, sec( t.seconds() )
, msec( t.mseconds() )
{}

/*******************************************************************
* Function Name: operator +
********************************************************************/
Time Time::operator +( const Time &t ) const
{
  if ( *this == Time::Inf || t == Time::Inf )
    return Time::Inf;

  Time st( hours() + t.hours(),
     minutes() + t.minutes(),
     seconds() + t.seconds(),
     mseconds() + t.mseconds() );

  //st.normalize();  // It's called by constructor and is not needed
  return st;
}

/*******************************************************************
* Function Name: opeartor -
********************************************************************/
Time Time::operator -( const Time &t ) const
{
  if ( *this == Time::Inf || t == Time::Inf )
    return Time::Inf;

  Time st( hours() - t.hours(),
     minutes() - t.minutes(),
     seconds() - t.seconds(),
     mseconds() - t.mseconds() );

  //st.normalize();  // It's called by constructor and is not needed
  return st; 
}

/*******************************************************************
* Function Name: opeartor ==
********************************************************************/
bool Time::operator ==( const Time &t ) const
{
  return ( hours()    == t.hours()    &&
     minutes()  == t.minutes()  &&
     seconds()  == t.seconds()  &&
     mseconds() == t.mseconds() );
}

//HENRY compile fix
bool Time::operator !=( const Time &t ) const
{
  return !((*this) == t);
}
/*******************************************************************
* Function Name: opeartor =
********************************************************************/
Time &Time::operator =( const Time &t )
{
  hour = t.hours();
  sec = t.seconds();
  min = t.minutes();
  msec = t.mseconds();
  return *this;
}

/*******************************************************************
* Function Name: operator <<
********************************************************************/
bool Time::operator <( const Time &t ) const
{
  return( hours() < t.hours()    ||
        ( hours() == t.hours() &&
         ( minutes() < t.minutes()  ||
          ( minutes() == t.minutes()  &&
            ( seconds() < t.seconds() ||
             ( seconds() == t.seconds() &&
              mseconds() < t.mseconds() ) ) ) ) ) ) ;
}

//HENRY compile fix
bool Time::operator >=( const Time &t ) const
{
  return !((*this) < t);
}

//HENRY compile fix
bool Time::operator <=( const Time &t ) const
{
  return (((*this) < t)||((*this) == t));
}

//HENRY compile fix
bool Time::operator >( const Time &t ) const
{
  return !((*this) <= t);
}

/*******************************************************************
* Function Name: asString
********************************************************************/
std::string Time::asString() const
{
  char buf[100];

  if (*this == Time::Inf)
    sprintf( buf, "..." );
  else
    sprintf( buf, "%02d:%02d:%02d:%03d", hours(), minutes(), seconds(), mseconds() );

  return std::string( buf );
}

/*******************************************************************
* Function Name: makeFrom
* Description: make from std::string
********************************************************************/
Time &Time::makeFrom( const std::string &str )
{
  sscanf( str.c_str(), "%d:%d:%d:%d", &hour, &min, &sec, &msec );
  normalize();
  return *this;
}

/*******************************************************************
* Function Name: makeFrom
* Description: make from miliseconds
********************************************************************/
Time &Time::makeFrom( const clock_t miliseconds )
{
  clock_t milsec = miliseconds;
  minutes( static_cast< int >( milsec/ (1000*60) ) ) ;
  milsec -= (minutes()*1000*60);
  seconds( static_cast< int >( milsec/1000 ) ) ;
  milsec -= (seconds()*1000);
  mseconds( static_cast< int >( milsec) );
  normalize() ;
  return *this ;
}

/*******************************************************************
* Function Name: normalize
********************************************************************/
Time &Time::normalize()
{
  // 0..999 Miliseconds
  adjust( msec, sec , 1000 );

  // 0..59 seconds
  adjust( sec , min , 60   );

  // 0..59 minutes
  adjust( min , hour, 60   );

  return *this ;
}

/*******************************************************************
* Function Name: adjust
********************************************************************/
Time &Time::adjust( int &left, int &right, int maxVal )
{
  if( left >= maxVal )
  {
    right += left / maxVal ;
    left %= maxVal ;
  } else if( left < 0 )
  {
    register int aux = abs( left / maxVal ) + ( left % maxVal == 0 ? 0 : 1 );
    right -= aux ;
    left += maxVal * aux ;
  }
  return *this ;
}

/*******************************************************************
* Function Name: currentTime
********************************************************************/
/* static */
Time Time::currentTime()
{
  clock_t miliseconds;

  const clock_t clksPerMilisec_c = CLOCKS_PER_SEC/1000;

  gettimeofday(&curtv,NULL); //get the current time
    
  //gettimeofday() returns microseconds, which is 1/1000 of miliseconds 
  //and 1/1,000,000 of seconds. And CLOCKS_PER_SEC = 1,000,000
  //We want to convert ticks_now from microseconds into miliseconds.

//Mohammad bug fix
  miliseconds = (curtv.tv_sec - starttv.tv_sec)*1000 + (curtv.tv_usec - starttv.tv_usec)/clksPerMilisec_c ;
  Time st( miliseconds );       // create from miliseconds a Time

  return st;

}//currentTime

