/*******************************************************************
*
*  DESCRIPTION: class Coordinator
*
*  AUTHOR:    Amir Barylko & Jorge Beyoglonian 
*  Version 2: Daniel A. Rodriguez
*  Version 3: Henry Yu
*
*  EMAIL: mailto://amir@dc.uba.ar
*         mailto://jbeyoglo@dc.uba.ar
*         mailto://drodrigu@dc.uba.ar
*
*  DATE: 27/6/1998
*  DATE: 04/6/1999 (v2)
*  DATE: 10/4/2004 (v3)
*
*******************************************************************/

#ifndef __COORDINATOR_H
#define __COORDINATOR_H

/** include files **/
#include "process.h"          // class Processor
#include <set>
#include <list>
#include <map>

/** forward declarations **/
class Coupled ;

/** declarations **/
typedef std::list<ProcId> ImminentChildren;
typedef std::set<ModelId, std::less<ModelId> > SyncAtomicsSet;
typedef std::set<ModelId, std::less<ModelId> > StructuredSet;
typedef std::map<ProcId, int, std::less<ProcId> > SCReq;
typedef std::set<ModelId, std::less<ModelId> > WaitForDoneQueue;

class Coordinator: public Processor 
{
public:
  enum WaitingMode
  {
    noWaiting_c = 0,
    waitingUponRecvInitMsg_c,	
    waitingUponRecvCollectMsg_c,	
    waitingUponRecvInternalMsg_c,
    waitingUponRecvOutputMsg_c,		//not used
    waitingUponRecvStrucChangeMsg_c,
    waitingUponRecvSelfStrucChange_c,
    waitingUponRecvStartMsg_c,
  };

  Coordinator( Coupled * ); //constructor
  ~Coordinator();

  Processor &receive( const InitMessage & ) ;

  Processor &receive( const InternalMessage & ) ;

  Processor &receive( const OutputMessage & ) ;

  Processor &receive( const ExternalMessage & ) ;

  Processor &receive( const DoneMessage & ) ;

  Processor &receive( const CollectMessage & ) ;

  Processor &receive( const StartMessage &) ;

  void printSyncSet_v();	//debug

  void printStrucSet_v();

protected:
  Coordinator(const Coordinator & ) ;  // Copy constructor

  Coordinator& operator=(const Coordinator & )  ;   // Assignment operator

  int operator==(const Coordinator & ) const ;  // Equality operator

  Processor &id( const ProcId &pid ) ;

  const ProcId imminentChild() const; 

  const ImminentChildren &immiChildren() const;
  const SyncAtomicsSet &syncSet() const;
  const StructuredSet &structuredset() const;
  const SCReq &screquest() const;

  //keep this function for backward compatibility
  Coordinator &inminentChild( const ProcId & );  

  long doneCount_m;
  long receivedX_m;

private:
  ProcId inminent ;
  SyncAtomicsSet syncSet_m;
  StructuredSet strucset;
  WaitForDoneQueue collectWaitForDoneQ_m;
  //WaitForDoneQueue outputWaitForDoneQ_m;
  WaitingMode waitingMode_m;
  SCReq screq;
  ImminentChildren immiChildren_m;
  
  Coordinator &recalcImminentChildren() ;

};  // class Coordinator


/** inline **/

//keep this function for backward compatibility
inline Coordinator &Coordinator::inminentChild( const ProcId &mid )
{
  inminent = mid ;
  return *this ;
}

inline const ProcId Coordinator::imminentChild() const
{
  return *(immiChildren_m.begin());
}


inline const ImminentChildren &Coordinator::immiChildren() const 
{
  return immiChildren_m;
}

inline const SyncAtomicsSet &Coordinator::syncSet() const 
{
  return syncSet_m;
}
inline const StructuredSet &Coordinator::structuredset() const
{
  return strucset;
}

inline const SCReq &Coordinator::screquest() const
{
  return screq;
}
#endif   //__COORDINATOR_H 
