/*******************************************************************
*
*  DESCRIPTION: class Coupled
*
*  AUTHOR:    Amir Barylko & Jorge Beyoglonian 
*  VERSION 2: Daniel Rodriguez
*
*  EMAIL: mailto://amir@dc.uba.ar
*         mailto://jbeyoglo@dc.uba.ar
*         mailto://drodrigu@dc.uba.ar
*
*  DATE: 27/6/1998
*  DATE: 27/2/1999 (v2)
*
*******************************************************************/

/** include files **/
#include <algorithm>
#include "coupled.h"    // base header
#include "procadm.h"  // SingleProcessorAdmin

/** public functions **/
/*******************************************************************
* Function Name: addModel
* Description: add a model to the list of children
********************************************************************/
Model &Coupled::addModel( Model &mod )
{
  //std::cout<<"new model is added to its parent"<<std::endl;
  mod.parentId( id() );

  ModelList::iterator cursor = find_if( childs.begin(), childs.end(), 
    std::bind2nd( std::greater<ModelId>(),  mod.id() ) );

  childs.insert( cursor, mod.id() );
  childs.unique();
  if(mod.mType() == Model::revatomicType)
    execId = mod.id();

  return *this;
}

/*******************************************************************
* Function Name: addInfluence
* Description: add a link between two ports
********************************************************************/
Model &Coupled::addInfluence( const std::string &sourceName, const std::string &sourcePort, const std::string &destName, const std::string &destPort) 
{
  Port &port1( SingleProcessorAdmin::Instance().processor( sourceName ).model().port( sourcePort ) );
  Port &port2( SingleProcessorAdmin::Instance().processor( destName ).model().port( destPort ) );

  port1.addInfluence( port2 );
  return *this;
}
Model &Coupled::delModelId(ModelId mid) // ModelId is the atomic model id which is removed
{
  std::cout<<"(Coupled delModelId) ModelId is "<<mid<<std::endl;
  childs.remove(mid);
  ModelMap::iterator it = modelmap.begin();
  while(it != modelmap.end())
    {
      if (it->second == mid) 
        {
	  Processor &proc = SingleProcessorAdmin::Instance().processor(it->first);
	  Model &mdl = proc.model();
	  mdl.parentId(0);
          modelmap.erase(it);
	 break;
        }
      else it++;
    }

  return *this;
}
Model &Coupled::addModelId(ModelId mid)
{
  childs.push_back(mid);
  Processor &proc = SingleProcessorAdmin::Instance().processor(mid);
  Model &mdl = proc.model();
  std::string mName = mdl.description();
  modelmap[mName] = mid;
  return *this;
}
Model &Coupled::print(std::ostream &os, int indent /* = 0 */)
{
  MASSERT(&os);
  std::string indentStr = "";
  for (int i = 0; i < indent; i++)
  {
    indentStr += " ";
  }//for

  os << indentStr << "Coupled Begin: " << asString() << std::endl;
  Model::print(os, indent);
  ModelList::iterator cursor;
  os << indentStr << "  Children Begin" << std::endl;

  for(cursor = childs.begin(); cursor != childs.end(); cursor++)
  {
    Model &m = SingleProcessorAdmin::Instance().processor(*cursor).model();
    os << indentStr << "    Child: " << m.asString();
    if (m.className() == COUPLED_NAME) { 
      os << " (coupled)" << std::endl;
      m.print(os, indent+6);
    }else{
      os << " (atomic)" << std::endl;
      m.print(os, indent+4);
    }//if-else
  }//for

  os << indentStr << "  Children End" << std::endl;
  os << indentStr << "Coupled End: " << asString() << std::endl;

  return *this;
}//print()
void Coupled::setnewmodels()
{
  newmodels.clear();
  int flag = 0;
  //newmodels = childs;
  ModelList::iterator ncursor, ocursor;
  printoldchildren();
  printchilds();
  ncursor = childs.begin();
  ocursor = oldchilds.begin();
  while(ncursor != childs.end())
    {
      while(ocursor != oldchilds.end())
	{
	  if((*ncursor) == (*ocursor)) {flag = 1;break;}
	  else  ocursor++;
	}
      if((flag == 0) && (ocursor == oldchilds.end())) newmodels.push_back(*ncursor);
      flag = 0;
      ocursor = oldchilds.begin();
      ncursor++;
    }
  oldchilds.clear();
  oldchilds = childs;
  return;
}
void Coupled::setremmodels()
{
  removedmodels.clear();
int flag = 0;
  //newmodels = childs;
  ModelList::iterator ncursor, ocursor;
  ncursor = childs.begin();
  ocursor = oldchilds.begin();
  while(ocursor != oldchilds.end())
    {
      while(ncursor != childs.end())
	{
	  if((*ncursor) == (*ocursor)) {flag = 1;break;}
	  else  ncursor++;
	}
      if((flag == 0) && (ncursor == childs.end())) removedmodels.push_back(*ocursor);
      flag = 0;
      ncursor = childs.begin();
      ocursor++;
    }
  return;
}

void Coupled::printoldchildren()
{
  std::cout<<"print oldchilds"<<std::endl;
  ModelList::const_iterator it(oldchilds.begin());
  std::cout<<"old children list is: "<<std::endl;
  for(; it != oldchilds.end(); it++)
    std::cout<<(*it)<<std::endl;   
}
void Coupled::printchilds()
{
  std::cout<<"print childs"<<std::endl;
  ModelList::const_iterator it(childs.begin());
  std::cout<<" children list is: "<<std::endl;
  for(; it != childs.end(); it++)
   std::cout<<(*it)<<std::endl;   
}
