/*******************************************************************
*
*  DESCRIPTION: class Coupled
*
*  AUTHOR:    Amir Barylko & Jorge Beyoglonian 
*  Version 2: Daniel Rodriguez
*
*  EMAIL: mailto://amir@dc.uba.ar
*         mailto://jbeyoglo@dc.uba.ar
*         mailto://drodrigu@dc.uba.ar
*
*  DATE: 27/6/1998
*  DATE: 27/2/1999 (v2)
*
*******************************************************************/

#ifndef __COUPLED_H
#define __COUPLED_H

/** include files **/
#include <list>    // Template list
#include "model.h"   // Base class Model
#include "mainsimu.h"

/** foward declarations **/
class SingleModelAdmin;
class MainSimulator ;

#define  COUPLED_NAME  "Coupled"

/** definitions **/
class Coupled : public Model 
{
public:
  Model &addModel( Model & );

  Model::ModelType mType() { return Model::coupledType; }

  typedef std::list<ModelId> ModelList;

  const ModelList &children() const
      {return childs;}
  void oldchildren()
    {oldchilds.clear(); oldchilds = childs;}
  void printoldchildren();
  void printchilds();
  void setnewmodels();
  void setremmodels();
  const ModelList &getnewmodels() const
    {return newmodels;}

  void setvalue(int value)
    {   strucvalue = value; }
  int getvalue()
    {return strucvalue;}

  const ModelList &getremmodels() const
    {return removedmodels;}
 
  typedef std::map<std::string, ModelId, std::less<std::string> > ModelMap;
  Model &delModelId(ModelId mid);
  Model &addModelId(ModelId mid);

  ModelId exective()
    {return execId;}

  enum Type
  {
    cell,
    regular
  } ;

  virtual Type type() const
      {return regular;}
  
  virtual std::string className() const
      {return COUPLED_NAME;}
  ModelMap &getmodelmap()
   { return modelmap; }
protected:
  friend class SingleModelAdmin ;
  friend class ModelAdmin ;
  friend class MainSimulator ;

  Coupled( const std::string &name = "Coupled" )  // Default constructor
      : Model( name )
      {}

  Coupled( const Coupled & )  ;     // Copy constructor

  virtual Model &addInfluence( const std::string &sourceName, const std::string &sourcePort, const std::string &destName, const std::string &destPort) ;

  virtual Model &print(std::ostream &os, int indent = 0);

private:
  ModelList childs, oldchilds ;
  ModelMap modelmap;
  ModelList newmodels, removedmodels;
  int strucvalue;

} ; // class Coupled


#endif   //__COUPLED_H 
