/*******************************************************************
*
*  DESCRIPTION: struct Event
*
*  AUTHOR: Ezequiel Glinsky
*
*  EMAIL: mailto://eglinsky@dc.uba.ar
*
*  DATE: 04/09/2001
*
*******************************************************************/

#ifndef __DEADLINE_H
#define __DEADLINE_H

/** include files **/
#include "cdTime.h"    // class Time
#include "port.h"       // class Port
#include "except.h"       // class MASSERT

/** foward declarations **/

/** definitions **/
struct Deadline
{
  Time timeDeadline ;    
  const Port *port ;
  const Port *outport ;    

  Deadline( const Time t = Time::Zero, 
            const Port *p = NULL, 
	    const Port *pout = NULL);

  bool operator < ( const Deadline &dl )
    {return timeDeadline < dl.timeDeadline;}
    
  Deadline &operator =( const Deadline &dl ) ;

  const std::string asString() const
    {return timeDeadline.asString() + " / " + port->asString() + " / " + outport->asString();}

  const Port &getPort() const
  {MASSERT(port); return *port;}

  const Port &getOutport() const
  {MASSERT(outport); return *outport;}
            
} ;

/** inline methods **/
inline
Deadline::Deadline( const Time t, const Port *p, const Port *pout)
: timeDeadline( t )
, port( p )
, outport( pout )
{}

inline
Deadline &Deadline::operator =( const Deadline &dl )
{
  timeDeadline = dl.timeDeadline ;    
  port = dl.port ;
  outport = dl.outport ;    
  return *this;
}

#endif   //__DEADLINE_H
