/*******************************************************************
*
*  DESCRIPTION: class FlatDEVSCoordinator, FlatDEVSCoordinatorState
*
*  AUTHOR:    Ezequiel Glinsky
*
*  EMAIL: mailto://eglinsky@dc.uba.ar
*
*  DATE: 11/2001
*
*******************************************************************/

#ifndef __FLATDEVSCOORDINATOR_H
#define __FLATDEVSCOORDINATOR_H

/** include files **/
#include <set>
#include "process.h"         // class Processor
//#include "flatdcst.h" // class FlatDEVSCoordinatorState
//#include "atomdata.h" // class AtomicData
#include "coordin.h" //class Coordinator ImminentChildren SyncAtomicsSet

#include <map>
#include <string>      // Template string
#include "strutil.h"    // hash<string>


/** forward declarations **/
class Coupled ;
//class AtomicData ;

/** declarations **/
class FlatDEVSCoordinator: public Processor
{
public:

  enum WaitingMode
  {
    noWaiting_c = 0,
    waitingUponRecvInitMsg_c,
    waitingUponRecvCollectMsg_c,
    waitingUponRecvInternalMsg_c,
    waitingUponRecvOutputMsg_c,         //not used
    waitingUponRecvStrucChangeMsg_c,
    waitingUponRecvSelfStrucChange_c,
    waitingUponRecvStartMsg_c,
  };

  FlatDEVSCoordinator( Coupled * ); //constructor
  //FlatDEVSCoordinator( ); //constructor

  ~FlatDEVSCoordinator(); //destructor

  Processor &receive( const InitMessage & );
  Processor &receive( const CollectMessage & );
  Processor &receive( const InternalMessage & );
  Processor &receive( const ExternalMessage & );
  Processor &receive( const OutputMessage & );
  Processor &receive( const DoneMessage & );

  const Time &nextChange() const;
  const Time &lastChange() const;
  inline const Time absoluteNext() const {
    return nextChange() + lastChange();
  }

  bool isAtomic();

  void printSyncSet_v();        //debug

protected:

  ProcId parentId;

  FlatDEVSCoordinator(const FlatDEVSCoordinator & );  // Copy constructor

  FlatDEVSCoordinator& operator=(const FlatDEVSCoordinator & );   // Assignment operator

  int operator==(const FlatDEVSCoordinator & ) const;  // Equality operator

  FlatDEVSCoordinator &nextChange( const Time & );
  FlatDEVSCoordinator &lastChange( const Time & );

  Processor &id( const ProcId &pid);

  const ProcId imminentChild() const;
  const ImminentChildren &immiChildren() const;
  const SyncAtomicsSet &syncSet() const;
  const StructuredSet &structuredset() const;
  const SCReq &screquest() const;
  
private:
  Time next ;  // These are NOT part of the ParProcessorState in the FlatDEVSCoordinator
  Time last ;  // These are NOT part of the ParProcessorState in the FlatDEVSCoordinator
  long doneCount_m;
  // The SyncAtomicsSet synchronizeSet in FlatDEVSCoordinator 
  // stores the children that must be synchronized at any given time
  SyncAtomicsSet syncSet_m;
  StructuredSet strucset;
  SCReq screq;
  WaitForDoneQueue collectWaitForDoneQ_m;
  WaitingMode waitingMode_m;
  ImminentChildren immiChildren_m;
  FlatDEVSCoordinator &recalcImminentChildren() ;

};  // class FlatDEVSCoordinator

/*******************************************************************
* Function Name: Shortcuts to state variables
********************************************************************/
inline
  const Time &FlatDEVSCoordinator::nextChange() const
{
  return next;
}

inline
  const Time &FlatDEVSCoordinator::lastChange() const
{
  return last;
}

inline
FlatDEVSCoordinator &FlatDEVSCoordinator::nextChange( const Time &t ){
  next = t;
  return *this;
}

inline
FlatDEVSCoordinator &FlatDEVSCoordinator::lastChange( const Time &t ){
  last = t;
  return *this;
}

inline const ProcId FlatDEVSCoordinator::imminentChild() const
{
  return *(immiChildren_m.begin());
}

inline const ImminentChildren &FlatDEVSCoordinator::immiChildren() const
{
  return immiChildren_m;
}

inline const SyncAtomicsSet &FlatDEVSCoordinator::syncSet() const
{
  return syncSet_m;
}
inline const StructuredSet &FlatDEVSCoordinator::structuredset() const
{
  return strucset;
}

inline const SCReq &FlatDEVSCoordinator::screquest() const
{
  return screq;
}
#endif   //__FLATDEVSCOORDINATOR_H
