/*******************************************************************
*
*  DESCRIPTION: class Ini
*
*  AUTHOR:    Amir Barylko & Jorge Beyoglonian 
*  Version 2: Daniel A. Rodriguez
*
*  EMAIL: mailto://amir@dc.uba.ar
*         mailto://jbeyoglo@dc.uba.ar
*         mailto://drodrigu@dc.uba.ar
*
*  DATE: 27/6/1998
*  DATE:  4/6/1999 (v2)
*
*******************************************************************/

/** include files **/
#include <fstream>
#include "ini.h"
#include "strutil.h"  // lowerCase
#include "prnutil.h"  // operator << list< T >
#include <string>

/** public functions **/
const Ini::StDefList &Ini::strucgroup(const std::string &groupName, const std::string &SComm) const
{

  std::string lower(groupName);

  //std::cout<<"group name is "<<groupName<<", scomm is "<<SComm<<std::endl;
  StGroupList::const_iterator sit = stgroups.begin();
  while (sit != stgroups.end())
    {
      unsigned int found;
      
      found = (sit->first).find(groupName, 0);
      //std::cout<<"the current group is "<<sit->first<<", found is "<<found<<std::endl;
      if (found >= 0)
	{

	  StDefList::const_iterator dit = sit->second.find("scomm");
	  std::string dest = dit->second.front();
	  if(SComm.compare(dest) == 0) 
           {
	     //std::cout<<"dest is "<<dest<<std::endl;
               return sit->second;
	   }
   
	}
      sit++;
    }

  if ( sit == stgroups.end())
    {
      IniRequestException e( groupName + "Not Found !");
      MTHROW( e );
    }
}
/*******************************************************************
* Function Name: group
********************************************************************/
const Ini::DefList &Ini::group( const std::string &groupName ) const
{
  std::string lower( groupName ) ;
  std::cout<<"group name is "<<groupName<<std::endl;

  if( groups.find( lower ) == groups.end() )
  {
    IniRequestException e( groupName + " Not Found!" ) ;
    MTHROW( e );
  }
  return groups.find( lower )->second;
}

/*******************************************************************
* Function Name: definition
* Description: ids asociados a la definicion
********************************************************************/
const Ini::IdList &Ini::definition( const std::string &groupName, const std::string &defName ) const
{
  const DefList &dlist( this->group( lowerCase( groupName ) ) ) ;

  if( dlist.find( lowerCase( defName ) ) == dlist.end() )
  {
    IniRequestException e( groupName + "/" + defName + " Not Found!" ) ;
    MTHROW( e );
  }
  return dlist.find( lowerCase( defName ) )->second ;
}

const Ini::IdList &Ini::stdefinition( const std::string &stgroupName, const std::string &SComm, const std::string &stdefName) const
{
  // std::cout<<"(Ini)received pair is "<<stgroupName<<" "<<SComm<<std::endl;

  const StDefList &dlist(this->strucgroup(stgroupName,SComm ) ) ;
  if( dlist.find(lowerCase(stdefName)) == dlist.end())
    {
      IniRequestException e( stgroupName + "/" + stdefName + " Not Found!" ) ;
      MTHROW( e );
    }
  return dlist.find( lowerCase(stdefName))->second;
}
/*******************************************************************
* Function Name: parse
* Description: llama a parse con un istd::fstream
********************************************************************/
Ini &Ini::parse( const std::string &fileIn, bool isFlatDEVS /* = false */)
{
  std::cout<<"enter parse function "<<isFlatDEVS<<"file name is "<<fileIn<<std::endl;
  std::ifstream in( fileIn.c_str() );
  return this->parse( in, isFlatDEVS );
}

/*******************************************************************
* Function Name: parse
* Description: parsea el inifile
********************************************************************/
Ini &Ini::parse( istream & in, bool isFlatDEVS )
{
  std::cout<<"begin to parse "<<std::endl;
  std::string actualGroup, actualDef ;
  int struc = 0;

  TokenIterator cursor( in ) ;

  while( cursor != TokenIterator() )
  {
    std::string word ;
    Token tokenId = this->readToken( cursor, word ) ;
    //std::cout<<"current token is "<<word<<"token id is "<<tokenId<<std::endl;
    switch( tokenId )
    {
      case group_TK:
      {
        if((word == "top") && isFlatDEVS) {
          actualGroup = "flattop";
	}else{
	  unsigned int found;
	  found = word.find("update", 0);
	  if(found <= word.size())
           {
              struc = 1;
              std::string str = word.substr(0, found);
	      //std::cout<<"the group_TK token is "<<word<<" the substr is "<<str<<std::endl; 
	      if((str == "top") && isFlatDEVS){
		actualGroup = "flat" + word;
	      }     
	      else{
               actualGroup = word;
	      }
               stgroups[actualGroup];
               this->print_c();
		 
		}
	   else{
	     //std::cout<<"current token is not a structure component "<<std::endl;
	    struc = 0;
            actualGroup = word;
	    //  filestr<<"the token is "<<word<<std::endl;
            groups[ actualGroup ];
            this->print_v();
            }
         
        }
        
        break;
      }
      case def_TK:
      {
	if(struc){
        actualDef = word ;
        stgroups[ actualGroup ][ actualDef ] ;
	}
	else{
           actualDef = word ;
           groups[ actualGroup ][ actualDef ] ;
	}
      
        break;
      }
      case id_TK:
      {
	if(struc)
         {
	    //  filestr <<" This is the structural ID "<<std::endl;
	    stgroups[actualGroup][actualDef].push_back(word);
	  }
	else{
            groups[ actualGroup ][ actualDef ].push_back( word ) ;
	}
        break;
      }
    }
  }
 // The following is used to adjust StGroupList.
  std::string group_name, stgroup_name, comp_lable, last_lable;
  last_lable = "";
  StGroupList::const_iterator stit(stgroups.begin());
 
  while(stit != stgroups.end())
    {
      stgroup_name = stit->first;
      unsigned int found;
      found = stgroup_name.find("update",0);
      if (found <= stgroup_name.size())
      {
        comp_lable = stgroup_name.substr(0, found);
	if(comp_lable != last_lable)
	  {
	last_lable = comp_lable;
	int find = 0;
        GroupList::const_iterator git(groups.begin());
	while(git != groups.end())
	  {
	    if(comp_lable == git->first) {find = 1; break;}
	    else git++;
	  }
	if(find){
	  // std::cout<<"(Ini ************)current git is "<<git->first<<std::endl;
           InsertData(git); RemoveStData(git);}
	//	else filestr<<group_name<<" is not found. "<<endl;
		 
        }
      }

      
      stit++;
    }
  ScommMap();
  // this->print_v();
  // this->print_c();
  return *this;   
}

/*******************************************************************
* Function Name: exists
********************************************************************/
bool Ini::exists( const std::string &groupName ) const
{
  return groups.find( lowerCase( groupName ) ) != groups.end() ;
}

/*******************************************************************
* Function Name: exists
********************************************************************/
bool Ini::exists( const std::string &groupName, const std::string &defName ) const
{
  GroupList::const_iterator gcursor( groups.find( lowerCase( groupName ) ) ) ;

  if( gcursor == groups.end() )
    return false ;

  DefList::const_iterator dcursor( gcursor->second.find( lowerCase( defName ) ) ) ; 

  return dcursor != gcursor->second.end();
}

/*******************************************************************
* Function Name: addGroup
* Description: add group to ini
********************************************************************/
Ini &Ini::addGroup( const std::string &groupName )
{
  groups[ lowerCase( groupName ) ] ;
  return *this ;
}

/*******************************************************************
* Function Name: addDefinition
********************************************************************/
Ini &Ini::addDefinition( const std::string &groupName, const std::string &def )
{
  groups[ lowerCase( groupName ) ][ lowerCase( def ) ] ;
  return *this ;
}

/*******************************************************************
* Function Name: addId
********************************************************************/
Ini &Ini::addId( const std::string &groupName, const std::string &def, const std::string &id )
{
  groups[ lowerCase( groupName ) ][ lowerCase( def ) ].push_back( lowerCase( id ) ) ;
  return *this ;
}

/*******************************************************************
* Function Name: save
********************************************************************/
Ini &Ini::save( const std::string &fileName )
{
  std::ofstream out( fileName.c_str() ) ;
  this->save( out ) ;
  return *this;
}

/*******************************************************************
* Function Name: save
********************************************************************/
Ini &Ini::save( std::ostream &out )
{
  Ini::GroupList::iterator cursor ;

  for( cursor = groups.begin() ; cursor != groups.end() ; cursor ++ )
  {
    out << '[' << cursor->first << ']' << std::endl ;
    out << cursor->second << std::endl ;   
  }
  return *this ;
}

/*******************************************************************
* Function Name: readToken
* Description: tokenize words
********************************************************************/
Ini::Token Ini::readToken( TokenIterator &cursor, std::string &word )
{
  if( (*cursor)[0] == '[' )
  {
    word = lowerCase( cursor->substr(1, cursor->size() - 2) );
    cursor++ ;
    return group_TK ;
  }
  word = lowerCase( *cursor );
  cursor ++ ;

  if( *cursor == ":" )
  {
    cursor ++ ;
    return def_TK ;
  } else
    return id_TK ;
}

/*******************************************************************
* Function Name: join
********************************************************************/
std::string Ini::join( IdList &id )
{
  std::string  s("");

  for (Ini::IdList::const_iterator cursor = id.begin(); cursor != id.end(); cursor++)
    s += *cursor;
  return s;
}

/*******************************************************************
* Function Name: print_v
********************************************************************/
Ini &Ini::print_v( )
{
  std::cerr << "\n********************************************\n";
  std::cerr << "Contents of Ini:\n";

  for (Ini::GroupList::const_iterator gcursor = groups.begin(); 
                                      gcursor != groups.end(); gcursor++)
  {
    std::cerr << gcursor->first << " (";
    for ( Ini::DefList::const_iterator dcursor = gcursor->second.begin(); 
                              dcursor != gcursor->second.end(); dcursor++)
    {
      std::cerr << dcursor->first << ", ";
    }//for
    
    std::cerr << ")\n";

  }//for

  return *this;
}//print_v

Ini &Ini::print_c( )
{
  std::cout << "\n********************************************\n";
  std::cout << "Contents of Structure Ini:\n";

  for (Ini::StGroupList::const_iterator scgcursor = stgroups.begin(); 
                                      scgcursor != stgroups.end(); scgcursor++)
  {
    std::cout << scgcursor->first << " (";
    for ( Ini::StDefList::const_iterator dcursor = scgcursor->second.begin(); 
                              dcursor != scgcursor->second.end(); dcursor++)
    {
      std::cout << dcursor->first << "( ";
      for(Ini::StIdList::const_iterator icursor = dcursor->second.begin(); 
                               icursor != dcursor->second.end(); icursor++)
	{
	  std::cout<<(*icursor)<<",";
	}
      std::cout<<")";
    }//for
    
    std::cout << ")\n";

  
   }//for

  return *this;
}//print_c

Ini &Ini::InsertData (GroupList::const_iterator git)
{

     stgroups[git->first];
     DefList::const_iterator dit(git->second.begin());
	     
     while(dit != git->second.end())
	{
	 
	    stgroups[git->first][dit->first];
	    IdList::const_iterator Iit(dit->second.begin());
	    while (Iit != dit->second.end())
              {
		if(dit->first != "components")
		  stgroups[git->first][dit->first].push_back(*Iit);
		else{
		  int found;
		  found = (*Iit).find('#');
		  if(found < 0) stgroups[git->first][dit->first].push_back(*Iit);
		}
		Iit++;

	      }
	   
	   
	     dit++;
	}
				 
	return *this;
} 

Ini &Ini::RemoveStData(GroupList::const_iterator git)
{

 DefList::const_iterator dit(groups[git->first].begin());
  while (dit != groups[git->first].end())
    {
        if(dit->first == "scomm")
	  {
	    IdList::iterator it1(groups[git->first]["scomm"].begin());
	    IdList::iterator it2(groups[git->first]["scomm"].end());
	    groups[git->first]["scomm"].erase(it1,it2 );
		
            groups[git->first].erase("scomm");  
            break;
          }
	else 
         {
           dit++;
	   //  if(dit == groups[git->first].end())
	   //   filestr<<"SComm is not found."<<std::endl;
	 }
    }


  return *this;
}

Ini &Ini::ScommMap()
{
  std::cout<<"entering scommmap"<<std::endl;
  int count = 0;
  unsigned found;
  if(!stgroups.empty()){
  StGroupList::const_iterator it(stgroups.begin());
  std::string name = it->first;
  std::cout<<"(Ini SCommMap)name is "<<name<<std::endl;
  
  // unsigned int found = name.find("update", 0);
  // if (found >= 0) {name = name.substr(0, found);std::cout<<"name is "<<name<<std::endl;}
  stmap[ name ];
  std::string commName = stgroups[it->first]["scomm"].front();
  std::cout<<"commName is "<<commName<<std::endl;
  stmap[name][commName] = ++count;
std::cout<<"the value of commName is "<<stmap[name][commName]<<std::endl;
  it++;
    while ( it != stgroups.end())
      {
	std::string name1 = it->first;
	found = name1.find("update", 0);
	if(found >= 0) name1 = name1.substr(0,found);
        std::cout<<"name1 is "<<name1<<"name is "<<name<<std::endl;
	if (name != name1)
	  {
	    //count = 0;
	    stmap[name1];
	    commName = stgroups[it->first]["scomm"].front();
            std::cout<<"commName is "<<commName<<std::endl;
	    stmap[name1][commName] = ++count;
            std::cout<<"the value of commName is "<<stmap[name1][commName]<<std::endl;
	    name = name1;
	  }
	else
	  {
	    std::cout<<"in the same group"<<std::endl;
	    commName = stgroups[it->first]["scomm"].front();
            std::cout<<"commName is "<<commName<<std::endl;
	    stmap[name][commName] = ++count;
            std::cout<<"the value of commName is "<<stmap[name][commName]<<std::endl;
	  }
	it++;
      }
  }
  
  
  return *this;
}
