/*******************************************************************
*
*  DESCRIPTION: class Ini
*
*  AUTHOR:    Amir Barylko & Jorge Beyoglonian 
*  Version 2: Daniel A. Rodriguez
*
*  EMAIL: mailto://amir@dc.uba.ar
*         mailto://jbeyoglo@dc.uba.ar
*	  mailto://drodrigu@dc.uba.ar
*
*  DATE: 27/6/1998
*  DATE: 20/3/1999 (v2)
*
*******************************************************************/

#ifndef _INI_HPP
#define _INI_HPP

/** include files **/
#include <fstream>
#include <list>
#include <map>
#include <iterator>	//HENRY compile fix
#include "except.h"

/** forward declarations **/
using std::istream;

/** declarations **/
class Ini
{
public:
	Ini() ;
	//	~Ini();

	typedef std::list< std::string > IdList;
        typedef std::list< std::string > StIdList;
	typedef std::map< std::string, IdList, std::less< std::string > > DefList;
        typedef std::map< std::string, StIdList, std::less< std::string > > StDefList;
	typedef std::map< std::string, DefList, std::less< std::string > > GroupList;
	typedef std::map< std::string, StDefList, std::less< std::string > > StGroupList;

	typedef std::map< std::string, int, std::less<std::string> > CommList;
	typedef std::map< std::string, CommList, std::less<std::string> > StrucMap;
	std::string join( IdList & );

	Ini &parse( istream &, bool isFlatDEVS);
	Ini &parse( const std::string &, bool isFlatDEVS  = false );

	Ini &save( std::ostream & );
	Ini &save( const std::string & );

	const GroupList &groupList() const;
	const StGroupList &stgroupList() const;
	const StrucMap &getmap() const;
	const DefList &group( const std::string & ) const;
	const IdList &definition( const std::string &groupName, const std::string &defName ) const;
	const StIdList &stdefinition( const std::string &stgroupName,const std::string &SComm, const std::string &stdefName) const;	
        const StDefList &strucgroup( const std::string &groupName, const std::string &SComm) const;  // sh
	Ini &addGroup( const std::string &groupName ) ;
	Ini &addDefinition( const std::string &groupName, const std::string &def ) ;
	Ini &addId( const std::string &groupName, const std::string &def, const std::string &id ) ;
        Ini &ScommMap();
	bool exists( const std::string &groupName, const std::string &defName ) const ;
	bool exists( const std::string &groupName ) const ;

	Ini &print_v();  //debug
	Ini &print_c();
private:
	typedef std::istream_iterator<std::string> TokenIterator ;

	Ini( const Ini& ) ;
	GroupList groups;
	StGroupList stgroups;
	StrucMap stmap;
        Ini &InsertData (GroupList::const_iterator git);
        Ini & RemoveStData (GroupList::const_iterator git);
	enum Token
	{
		group_TK,
		def_TK,
		id_TK
	} ;

	Token readToken( TokenIterator &, std::string & ) ;
};


class IniException : public MException
{
protected:
	IniException( const std::string &str = "Ini Exception" ): MException( str )
	{} ;
};


class IniParseException : public IniException
{
public:
	IniParseException() : IniException( "Parsing Error in Ini File" )
	{};
};


class IniRequestException : public IniException
{
public:
	IniRequestException( const std::string &msg ) : IniException( "Data not found!!!\nR(etry), I(gnore), P(anic)?" ) 
	{
		this->addText( msg );
	};
};

/** inline **/
inline
Ini::Ini()
{};

inline
const Ini::GroupList &Ini::groupList() const
{
	return groups;
}
inline
const Ini::StGroupList &Ini::stgroupList() const
{
  return stgroups;
}

inline
const Ini::StrucMap &Ini::getmap() const
{
  return stmap;
}
#endif //  _INI_HPP
