/*******************************************************************
*
*  DESCRIPTION: CLASS SimLoader
*
*  AUTHOR:    Amir Barylko & Jorge Beyoglonian 
*  Version 2: Daniel Rodriguez.
*  Version 3: Mohammad Moallemi
*
*  EMAIL: mailto://amir@dc.uba.ar
*         mailto://jbeyoglo@dc.uba.ar
*         mailto://drodrigu@dc.uba.ar
*	  mailto://moallemi@sce.carleton.ca
*
*  DATE: 27/06/1998
*  DATE: 11/04/1999 (v2)
*  DATE: 10/09/2008 (v3)
*
*******************************************************************/

#ifndef __LOADER_H
#define __LOADER_H

/** include files **/
#include "cdTime.h"
#include "evaldeb.h"    // EvalDebug

/** forward declarations **/
using std::iostream  ;
class MainSimulator ;

/** declarations **/

class SimLoader
{
public:
  virtual bool isFlatDEVS() const = 0;
  virtual const bool isRealTimeRun() const = 0;
  virtual const bool isRealHardRun() const = 0; //v3
  virtual const bool EmptyEventFile() const = 0; //v3
  virtual const bool showWallClockTime() const = 0;

  virtual std::istream &modelsStream()
      {return *models;}
      
  virtual std::istream &eventsStream()
      {return *events;}

  virtual std::ostream &logStream()
      {return *log;}

  virtual std::ostream &outputStream()
      {return *output;}

  const Time &stopTime() const
      {return stop;}
  
  const Time &endeventTime() const // event's deadline
  	  {return endevent;}
      
  bool printParserInfo() const
      {return printParser;}

  virtual ~SimLoader(){}        //Destructor

protected:
  friend class MainSimulator ;

  // ** Instance variables ** //
  Time stop, endevent ;
  bool printParser ;
  std::istream *models ;
  std::istream *events ;
  std::ostream *log ;
  std::ostream *output ;


  // ** Methods ** //
  SimLoader()          //Default constructor
    : models( NULL )
    , events( NULL )
    , log( NULL )
    {  EvalDebug().Stream( NULL );
      EvalDebug().Active( false );
    }


  virtual SimLoader &loadData() = 0;
  virtual SimLoader &writeResults() = 0;

  SimLoader &stopTime( const Time &t )
      {stop = t; return *this;}
  
  SimLoader &endeventTime(const Time &eev) //saves the deadline for the event
  	  {endevent = eev; return *this;}

  SimLoader &printParserInfo (const bool pp)
      {printParser = pp; return *this;}

};  // class SimLoader

#endif   //__LOADER_H 
