/*******************************************************************
*
*  DESCRIPTION: class MainSimulator
*
*  AUTHOR:    Amir Barylko & Jorge Beyoglonian 
*  Version 2: Daniel A. Rodriguez
*  Version 3: Mohammad Moallemi ((Robot functions added))
*
*  EMAIL: mailto://amir@dc.uba.ar
*         mailto://jbeyoglo@dc.uba.ar
*         mailto://drodrigu@dc.uba.ar
*         mailto: moallemi@sce.carleton.ca
*
*  DATE: 27/6/1998
*  DATE: 12/6/1999 (v2)
*  DATE: 27/9/2008, 23/10/2008 (v3)
*
*******************************************************************/

#ifndef __MAINSIMULATOR_H
#define __MAINSIMULATOR_H

/** include files **/
#include <iostream>
#include <fstream>
#include <set>
#include "root.h"          // EventList 
#include "ini.h"          // class Ini
#include "loader.h"        // class SimLoader 
#include "ltranadm.h"      // class LocalTransAdmin
#include "revatomic.h"

/** forward declarations **/
class Model ;
class Coupled ;
class Atomic ;
class CoupledCell ;


/** declarations **/
class MainSimulator
{
public:
  //std::ofstream parselog;
  static MainSimulator &Instance() ;
  MainSimulator &run() ;

  MainSimulator &loader( SimLoader *l )
      {sloader = l; return *this;}
  typedef std::set< std::string > ComponentSet;
typedef struct
{
  std::string sourceName;
  std::string sourcePort;
  std::string destName;
  std::string destPort;
}MAINLINK;
 typedef std::list< MAINLINK > MLinkList;
 Ini &iniFile()
   {return ini;}
 const Ini &iniFile() const
   {return ini; }
 ComponentSet &GetComponentSet()
   {return nameset;}
 MLinkList &GetLinkList()
   {return mlinklist;}    

 bool AddComponent(std::string mName);
 bool RemoveComponent(std::string mName);
 bool AddLink(std::string sourceName, std::string sourcePort, std::string destName, std::string destPort);
 bool RemoveLink(std::string sourceName, std::string sourcePort, std::string destName, std::string destPort);
 bool splitString( const std::string &full, std::string &first, std::string &second, char separator );
  void registerNewAtomics() ;

  bool isFlatDEVS() { return loader()->isFlatDEVS();}

  std::ostream &outputStream()
      {return loader()->outputStream();}

  std::ostream &logStream()
      {return loader()->logStream();}

  std::ostream &evalDebugStream()
      {return EvalDebug().Stream();}
      
  bool evalDebug()
      {return EvalDebug().Active();}
      
  bool existsParameter( const std::string &modelName, const std::string &parameterName ) const
      {return iniFile().exists( modelName, parameterName );}

  const std::string getParameter( const std::string &modelName, const std::string &parameterName ) const ;

//Robot Functions
  /*const bool Spin_Motor_Clockwise(int speed);		//v3
  const bool Spin_Motor_CounterClockwise(int speed);	//v3
  const bool Spin_Motor_Stop();				//v3	
  const bool Turn_V_Left(int speed);			//v3
  const bool Turn_V_Right(int speed);			//v3*/
  bool InitializeHardware();				//v3
  bool CloseHardware();				//v3
  /*const bool UpdateHardware();				//v3
  const bool CatcheEvent(int&);				//v3*/
//


  SimLoader *loader()
      {return sloader;}

  Time elapsedTime();    // for RTDevs use
  //  bool IsStrucComponent(std::string ModelName);

private:
  friend class Coupled ;
  friend class Atomic ;

  bool touchvalue;	//touch sensor value			v3
  int sonarvalue;	//sonar sensor value in centimeters 	v3
  double convalue; 	//input event value			v3
  int key;		//keyboard input 			v3
  std::string inportName;
  std::string outportName;   //end of declarations

  // ** class variables ** //   
  static MainSimulator *instance ;

  // ** instance constants ** //
  const char separator ;
  const char *componentLabel ;
  const char *inPortsLabel ;
  const char *outPortsLabel ;
  const char *linksLabel ;

  // ** instance variables ** //
  Ini ini ;
  SimLoader *sloader ;
  ComponentSet nameset;
  MLinkList mlinklist;

  Time startTime_m;    // for RTDevs use, keeps the start of simulation time


  // ** Methods ** //

  MainSimulator() ;
  MainSimulator &loadModels( std::istream&, bool printParserInfo ) ; 
  MainSimulator &loadExternalEvents( std::istream& ) ;
  MainSimulator &log( std::ostream & ) ;
  MainSimulator &output( std::ostream & ) ;
  MainSimulator &loadModel( Coupled &, Ini &, bool ) ;
  MainSimulator &loadPorts( Coupled &, Ini & ) ;
  MainSimulator &loadComponents( Coupled &, Ini &, bool ) ;
  MainSimulator &loadLinks( Coupled &, Ini & ) ;
  MainSimulator &loadFlattenLinks( Coupled &, Ini & ) ;
  MainSimulator &updateOutLinks( Coupled &, Ini & ) ;
  MainSimulator &updateAtomicOutLinks( Atomic & ) ;
  MainSimulator &addFlattenedLinksToFlatTop( const std::string sourcePort, 
  					     const std::string destName,
  					     const std::string destPort);
  MainSimulator &loadCells( CoupledCell &, Ini &, bool ) ;
  MainSimulator &loadInitialCellValues( CoupledCell &, Ini & ) ;
  MainSimulator &loadInitialCellValuesFromFile( CoupledCell &parent, const std::string &fileName );
  MainSimulator &loadInitialCellValuesFromMapFile( CoupledCell &parent, const std::string &fileName );
  MainSimulator &loadDefaultTransitions( CoupledCell &, Ini &, bool ) ;
  MainSimulator &loadPortInTransitions( CoupledCell &, Ini &, bool ) ;
  MainSimulator &loadLocalZones( CoupledCell &, Ini &, bool ) ;

  MainSimulator &registerTransition(const LocalTransAdmin::Function &, Ini &, bool );
  MainSimulator &registerTransitionPortIn(const LocalTransAdmin::Function &, Ini &, bool, const std::string & );

  void showEvents( const EventList &events, std::ostream &out = std::cout ) ;

  //  Ini &iniFile()
  //   {return ini;}

  //  const Ini &iniFile() const
  //  {return ini;}
      
 

};  // class MainSimulator

/** inline **/
inline
MainSimulator &MainSimulator::Instance()
{
  if( !instance )
    instance = new MainSimulator() ;

  return *instance ;
}

#endif   //__MainSimulator_H 
