/*******************************************************************
*
*  DESCRIPTION: class Model
*
*  AUTHOR:    Amir Barylko & Jorge Beyoglonian 
*  Version 2: Daniel Rodriguez
*
*  EMAIL: mailto://amir@dc.uba.ar
*         mailto://jbeyoglo@dc.uba.ar
*         mailto://drodrigu@dc.uba.ar
*
*  DATE: 27/6/1998
*  DATE: 25/4/1999 (v2)
*
*******************************************************************/

/** include files **/
#include "model.h"      // class Model
#include "strutil.h"    // lowerCase
#include "mainsimu.h"

/** public functions **/
/*******************************************************************
* Function Name: Model
* Description: constructor default
********************************************************************/
Model::Model( const std::string &name )
: ident( Processor::InvalidId ),
  parent_id( Processor::InvalidId ),
  execId(Processor::InvalidId),
  descript( name )
{}

/*******************************************************************
* Function Name: ~Model
* Description: destructor
********************************************************************/
Model::~Model()
{
  PortList::iterator cursor = inputList.begin();
  for ( ; cursor != inputList.end() ; cursor++ )
    delete cursor->second;

  for ( cursor = outputList.begin(); cursor != outputList.end(); cursor++ )
    delete cursor->second;
}

/*******************************************************************
* Function Name: addInputPort
* Post: adds the port to the input std::list
********************************************************************/
Port &Model::addInputPort( const std::string &portName )
{
  Port *pPort = new Port( lowerCase( portName ), id() );
  inputPorts()[ pPort->id() ] = pPort;
  return *pPort;
}

/*******************************************************************
* Function Name: addOuputPort
* Description: adds the port to the output std::list 
********************************************************************/
Port &Model::addOutputPort( const std::string &portName )
{
  Port *pPort = new Port( lowerCase( portName ), id() );
  outputList[ pPort->id() ] = pPort;
  return *pPort;
}
Model &Model::delInputPort( const std::string &portName )
{
  std::string name( lowerCase(portName));
  PortList::iterator cursor = inputPorts().begin();
  for(; cursor != inputPorts().end() && cursor->second->name() != name; cursor++)
    if(cursor == inputPorts().end()) 
     std::cout<<"the input port "<<portName<<" is not found!"<<std::endl;
    else 
       {
	 delete cursor->second;
         inputPorts().erase(cursor->first);
	
	 }
return *this;
}

Model &Model::delOutputPort( const std::string &portName )
{
  std::cout<<"delOutputPort "<<portName<<std::endl;
  std::string name( lowerCase(portName));
  PortList::iterator cursor = outputPorts().begin();
  for(; cursor != outputPorts().end() && cursor->second->name() != name; cursor++)
    if(cursor == outputPorts().end()) 
      std::cout<<"the output port "<<portName<<" is not found!"<<std::endl;
    else 
       {
         delete cursor->second;
         inputPorts().erase(cursor->first);
	
	 }
  return *this;
  }
/*******************************************************************
* Function Name: port
* Description: Retorna el port solicitado
********************************************************************/
Port &Model::port( const std::string &n )
{
  std::string name( lowerCase( n ) ) ;

  PortList::iterator cursor = inputPorts().begin();

  for( ; cursor != inputPorts().end() && cursor->second->name() != name; cursor++ )
    ;

  if (cursor == inputPorts().end())
  {
    for (cursor = outputPorts().begin(); cursor != outputPorts().end() && cursor->second->name() != name ; cursor++ )
      ;

    if ( cursor == outputPorts().end() )
    {
      InvalidPortRequest e ;
      e.addText( name + " port not found!" ) ;
      MTHROW( e ) ;
    }
  }
  MASSERT( cursor->second );
  return *( cursor->second );
}

/*******************************************************************
* Function Name: nextChange
********************************************************************/
Model &Model::nextChange( const Time &t )
{
  SingleProcessorAdmin::Instance().processor( id() ).nextChange(t);
  return *this;
}

/*******************************************************************
* Function Name: lastChange
********************************************************************/
Model &Model::lastChange( const Time &t )
{
  SingleProcessorAdmin::Instance().processor( id() ).lastChange(t);
  return *this;
}

Model &Model::strucChange(int &scomm)
{
  std::cout<<"($$$$$model )structure change command is set "<<scomm<<std::endl;
  SingleProcessorAdmin::Instance().processor( id() ).strucChange(scomm);
  return *this;
}
/*******************************************************************
* Function Name: print
********************************************************************/
Model &Model::print(std::ostream &os, int indent /* = 0 */)
{
  MASSERT(&os);
  //os << "\nModel Begin: " << asString() << endl;
  std::string indentStr = "";
  for (int i = 0; i < indent; i++)
  {
    indentStr += " ";
  }//for
  os << indentStr << "  Model::inputList:\n"; 
  PortList::iterator cursor; 
  for (cursor = inputPorts().begin();cursor != inputPorts().end(); cursor++)
  {
    MASSERT( cursor->second );
    cursor->second->print(os, indent);
  }//for

  os << indentStr << "  Model::outputList:\n"; 
  for (cursor = outputPorts().begin();cursor != outputPorts().end(); cursor++)
  {
    MASSERT( cursor->second ); //cursor->second is "Port *" type
    cursor->second->print(os, indent);
  }//for

  //os << "Model End\n";
  return *this;
}//print()
