/*******************************************************************
*
*  DESCRIPTION: class ModelAdmin, class SingleModelAdm
*
*  AUTHOR: Amir Barylko & Jorge Beyoglonian 
*
*  EMAIL: mailto://amir@dc.uba.ar
*         mailto://jbeyoglo@dc.uba.ar
*
*  DATE: 27/6/1998
*
*******************************************************************/

/** include files **/
#include "modeladm.h"         // base header
#include "procadm.h"          // processor header
#include "strutil.h"          // lowerCase 
#include "idcell.h"           // InertialDelayCell
#include "tdcell.h"           // TransportDelayCell
#include "flatcoup.h"         // class FlatCoupledCell

/** private data **/
ModelAdmin *SingleModelAdm::instance( NULL ) ;

/** public functions **/

/*******************************************************************
* Function Name: ModelAdmin
********************************************************************/
ModelAdmin::ModelAdmin()
: typeCount( 0 )
{
  strucCount = 0;
}

/*******************************************************************
* Function Name: registerAtomic
* Description: register a new atomic type 
********************************************************************/
ModelAdmin::AtomicType ModelAdmin::registerAtomic( const NewFunction &f, const std::string &description )
{
  std::cout<<"current description is "<<description<<std::endl;
  this->kinds[ this->typeCount ] = f.duplicate() ;
  this->types[ lowerCase( description ) ] = this->typeCount ;
  this->typeCount++ ;
  return this->typeCount ;
}

ModelAdmin::RevAtomicType ModelAdmin::registerRevAtomic(const StrucFunction &f, const std::string &description)
{
  this->struckinds[this->strucCount] = f.duplicate();
  this->strucTypes[lowerCase(description)] = this->strucCount;
  this->strucCount++;
  return this->strucCount;
}
/*******************************************************************
* Function Name: newAtomic
* Description: make a new atomic model and store it in the bd
********************************************************************/
Atomic &ModelAdmin::newAtomic( const AtomicType &type, const std::string &modelName ) 
{
  AtomicKinds::iterator cursor( this->kinds.find( type ) ) ;

  if( cursor == this->kinds.end() )
  {
    InvalidAtomicTypeException e;
    e.addText( std::string( "The number " ) + type + " is an unregistered type" ) ;
    MTHROW( e ) ;
  }
  
  NewFunction *fn = cursor->second ;
  std::cout<<"function is found"<<std::endl;
  Atomic *atomic = (*fn)( modelName ) ;
  std::cout<<"new atomic model is created"<<std::endl;

  SingleProcessorAdmin::Instance().generateProcessor( atomic ) ;
  return *atomic ;
}

/*******************************************************************
* Function Name: newAtomic
* Description: search the AtomicType and call to the another newAtomic method
********************************************************************/
Atomic &ModelAdmin::newAtomic( const std::string &name, const std::string &modelName ) 
{
  AtomicTypes::iterator cursor( this->types.find( lowerCase( name ) ) ) ;

  if( cursor == this->types.end() )
  {
    InvalidAtomicTypeException e ;
    e.addText( name + " is an unregistered type!" ) ;
    MTHROW( e ) ;
  }
  return this->newAtomic( cursor->second, modelName ) ;
}

RevAtomic &ModelAdmin::newRevAtomic(const RevAtomicType &type, const std::string &modelName)
{
  RevAtomicKinds::iterator rev_cursor(this->struckinds.find(type));
  if(rev_cursor == this->struckinds.end())
    {
      InvalidAtomicTypeException e;
      e.addText(std::string("RevAtomic: The number ") + type + " is an unregistered type");
      MTHROW(e);
    }
 std::cout<<"build up a new revatomic function for "<<modelName<<std::endl;
  StrucFunction *fn = rev_cursor->second;
 std::cout<<"new revatomic function is "<<fn<<std::endl;
  RevAtomic *revatomic = (*fn)(modelName);

  Processor &proc = SingleProcessorAdmin::Instance().generateProcessor(revatomic);
  std::cout<<"revatomic object is created "<<revatomic<<"processor id is "<<proc.id()<<std::endl;
  return *revatomic;
}
RevAtomic &ModelAdmin::newRevAtomic(const std::string &name, const std::string &modelName)
{
  RevAtomicTypes::iterator rev_cursor(this->strucTypes.find(lowerCase(name)));
  if(rev_cursor == this->strucTypes.end())
    {
      InvalidAtomicTypeException e;
      e.addText(std::string("RevAtomic: ") + name + " is an unregistered type ");
      MTHROW(e);
    }
  return this->newRevAtomic(rev_cursor->second, modelName);
}
/*******************************************************************
* Function Name: newAtomicCell
********************************************************************/
AtomicCell &ModelAdmin::newAtomicCell( bool inertial, const std::string &modelName )
{
  AtomicCell *atomicCell ;

  if( inertial )
    atomicCell = new InertialDelayCell( modelName ) ;
  else
    atomicCell = new TransportDelayCell( modelName ) ;

  SingleProcessorAdmin::Instance().generateProcessor( atomicCell ) ;
  return *atomicCell ;
}

/*******************************************************************
* Function Name: newCoupled
* Description: make a new coupled model and store it in the bd
********************************************************************/
Coupled &ModelAdmin::newCoupled( const std::string &modelName ) 
{
  Coupled *coupled = new Coupled( modelName ) ;

  SingleProcessorAdmin::Instance().generateProcessor( coupled ) ;

  return *coupled ;
}

/*******************************************************************
* Function Name: newCoupledCell
********************************************************************/
CoupledCell &ModelAdmin::newCoupledCell( const std::string &modelName )
{
  CoupledCell *coupled = new CoupledCell( modelName ) ;

  SingleProcessorAdmin::Instance().generateProcessor( coupled ) ;

  return *coupled ;
}

/*******************************************************************
* Function Name: newFlatCoupledCell
********************************************************************/
FlatCoupledCell &ModelAdmin::newFlatCoupledCell( const std::string &modelName ) 
{
  FlatCoupledCell *coupled = new FlatCoupledCell( modelName ) ;

  SingleProcessorAdmin::Instance().generateProcessor( coupled ) ;

  return *coupled ;
}
