/*******************************************************************
*
*  DESCRIPTION: classModelAdmin and SingleModelAdmin
*
*  AUTHOR: Amir Barylko & Jorge Beyoglonian 
*
*  EMAIL: mailto://amir@dc.uba.ar
*         mailto://jbeyoglo@dc.uba.ar
*
*  DATE: 27/6/1998
*
*******************************************************************/

#ifndef __MODEL_ADMIN_H
#define __MODEL_ADMIN_H
using namespace std;
/** include files **/
#include <map>            // Template map
#include <string>           // Template string
#include <iostream>
#include "model.h"

/** foward declarations **/
class ModelAdmin;
class SingleModelAdm ;
class Atomic ;
class AtomicCell ;
class Coupled ;
class CoupledCell ;
//class Model ;
class ProcessorAdmin ;
class Root ;
class MainSimulator ;
class RevAtomic;
class FlatCoupledCell ;

/** definitions **/

struct NewFunction
{
  virtual Atomic *operator()( const std::string &  ) = 0 ;   
  virtual NewFunction *duplicate() const = 0 ;
};

template <class T> 
struct NewAtomicFunction : NewFunction
{
  Atomic *operator()( const std::string &name )
  {
    cout<<"creating model"<<name<<endl;
return new T( name ) ;}
  NewFunction *duplicate() const
  {return new NewAtomicFunction<T>() ;}
 
} ;

struct StrucFunction
{
  virtual RevAtomic * operator()(const std::string &) = 0;
  virtual StrucFunction *duplicate() const = 0;
};

template <class T>
struct StrucRevAtomicFunction : StrucFunction
{
  RevAtomic * operator()(const std::string &name)
  {
    RevAtomic *prevatomic = new T(name);
    return prevatomic;
  }
  StrucFunction *duplicate() const
  {return new StrucRevAtomicFunction<T>();}
};
class ModelAdmin
{
public:   
  typedef int AtomicType ;
  typedef int RevAtomicType;
  AtomicType registerAtomic( const NewFunction &f, const std::string & );
  RevAtomicType registerRevAtomic(const StrucFunction &f, const std::string &);
protected:
  friend class MainSimulator ;
  friend class RevAtomic;
  friend class Coordinator ;
  friend class ProcessorAdmin ;
  friend class Root ;
  friend class CoupledCell ;  // para crear los atomicCell

  ModelAdmin( const ModelAdmin & );  // Copy constructor
  ModelAdmin& operator =( const ModelAdmin & ) ; // Assignment operator
  int operator ==( const ModelAdmin & ) const;   // Equality operator

  Atomic &newAtomic( const AtomicType &, const std::string &modelName );
  Atomic &newAtomic( const std::string &typeNmae, const std::string &modelName );
  RevAtomic &newRevAtomic(const RevAtomicType &, const std::string &modelName );
  RevAtomic &newRevAtomic(const std::string &typeName, const std::string &modelName);

  AtomicCell &newAtomicCell( bool inertial = false, const std::string &modelName = "AtomicCell" ) ;

  Coupled &newCoupled( const std::string &modelName ) ;
  CoupledCell &newCoupledCell( const std::string &modelName ) ;
  FlatCoupledCell &newFlatCoupledCell( const std::string &modelName ) ;


private:
  friend class SingleModelAdm ;

  typedef std::map< std::string, AtomicType, std::less< std::string > > AtomicTypes ;
  typedef std::map< AtomicType, NewFunction*, std::less< AtomicType > > AtomicKinds ;
  typedef std::map< std::string, RevAtomicType, std::less< std::string> > RevAtomicTypes;
  typedef std::map< RevAtomicType, StrucFunction*, std::less<RevAtomicType> > RevAtomicKinds;
  AtomicKinds kinds  ;
  AtomicType  typeCount ;
  AtomicTypes types ;

  RevAtomicKinds struckinds;
  RevAtomicType strucCount;
  RevAtomicTypes strucTypes;
  
  ModelAdmin() ;   // Default constructor

public:
  const AtomicTypes &newTypes() const ;

};  // class ModelAdmin

class SingleModelAdm
{
public:
  static ModelAdmin &Instance() ;
private:
  static ModelAdmin *instance ;
  SingleModelAdm() ;
};  // SingleModelAdm

/** inline methods **/

inline
SingleModelAdm::SingleModelAdm()
{}

inline
ModelAdmin &SingleModelAdm::Instance()
{
  if( !instance )
    instance = new ModelAdmin() ;

  return *instance ;
}

#endif   //__MODEL_ADMIN_H 
