/*******************************************************************
*
*  DESCRIPTION: messageBag class
*
*  AUTHOR:    Amir Barylko & Jorge Beyoglonian 
*  Version 2: Daniel A. Rodriguez
*
*  EMAIL: mailto://amir@dc.uba.ar
*         mailto://jbeyoglo@dc.uba.ar
*         mailto://drodrigu@dc.uba.ar
*
*  DATE: 27/6/1998
*  DATE: 25/4/1999 (v2)
*
*******************************************************************/

#ifndef __MSGBAG_H
#define __MSGBAG_H

/** include files **/
#include <string>
#include <list>
#include <map>
//#include "time.h"

/** foward declarations **/
class ExternalMessage;
class Time;

/** definitions **/
class MessageBag
{

public:
        typedef std::list<const ExternalMessage*> MessageList;

private:
        typedef std::map< std::string, MessageList, std::less< std::string > > MessagesOnPort;

        MessagesOnPort msgs;
        int count;
        Time msgtime;

public:

        MessageBag();   //Default Constructor
        ~MessageBag();

        MessageBag &add( const ExternalMessage* );

        bool portHasMsgs( const std::string& portName ) const;

        const MessageList& msgsOnPort( const std::string& portName ) const;

        int size() const
        {return count;}

        MessageBag& eraseAll();

        void print_v();	//debug

        const Time& time() const
        {return msgtime;}

        struct iterator
        {
          iterator() {}
          ~iterator() {}
          iterator( const MessagesOnPort::const_iterator &begin, 
	  	    const MessagesOnPort::const_iterator &end, 
		    const MessageList* currentList);
          iterator( const MessagesOnPort::const_iterator &begin, 
	  	    const MessagesOnPort::const_iterator &end );

          iterator &operator ++(int) ;

          const ExternalMessage *operator *() const;

          bool operator ==( const iterator &it ) const;
          bool operator !=( const iterator &it ) const;
          iterator &operator=( const iterator &it );

          MessagesOnPort::const_iterator msgsxport, msgsEnd;
          const MessageList *currentList;
          MessageList::const_iterator msgs;
        };

        iterator begin() const;
        iterator end() const;

}; // MessageBag

#endif   //__MSGBAG_H 
