/*******************************************************************
*
*  DESCRIPTION: class Port
*
*  AUTHOR:    Amir Barylko & Jorge Beyoglonian 
*  Version 2: Daniel A. Rodriguez
*
*  EMAIL: mailto://amir@dc.uba.ar
*         mailto://jbeyoglo@dc.uba.ar
*         mailto://drodrigu@dc.uba.ar
*
*  DATE: 27/6/1998
*  DATE: 28/2/1999 (v2)
*
*******************************************************************/

/** include files **/
#include "port.h"    // class Port
#include "model.h"   // class Model
#include "root.h"   // class Root
#include "coupled.h"   // class Coupled
#include "mainsimu.h"

/** public **/

int Port::idCount( 0 );

Port &Port::addInfluence( const Influence &influence )
{
  InfluenceList::iterator cursor ;

  for( cursor = influenceList.begin() ; 
    cursor != influenceList.end() && (*cursor)->model() < influence.model() ; 
    cursor++ ) ;

  influenceList.insert( cursor, &influence ) ;

  return *this ;
}

Port &Port::delInfluence( const Influence &influence )
{
  InfluenceList::iterator cursor;

  for( cursor = influenceList.begin(); cursor != influenceList.end(); cursor++)
  {
    if ((*cursor)->model() == influence.model()){ 
      influenceList.erase( cursor );
      break;
    }//if
  }//for

  return *this;
}

bool Port::findInfluence( const Influence *influence )
{
  bool found = false;
  InfluenceList::iterator cursor ;

  for( cursor = influenceList.begin(); cursor != influenceList.end(); cursor++)
  {
    if ((*cursor)->model() == influence->model()) 
      found = true;
  }//for
  return found;
}

Port &Port::name( const std::string &n )
{
        pname = n ;
        return *this ;
}
 
Port::Port( const std::string &n, const ModelId &id )
: pid( Port::idCount ++ )
, pname( n )
, mid( id )
{}

const Port &Port::print (std::ostream &os, int indent /* = 0 */) const
{
  MASSERT(&os);
  std::string indentStr = "";
  for (int i = 0; i < indent; i++)
  {
    indentStr += " ";
  }//for

  os << indentStr << "    Port: " << asString() << "\tinfluLst: ";
  InfluenceList::const_iterator cursor ;
  for( cursor = influenceList.begin(); cursor != influenceList.end(); cursor++)
  {
    Model &m = SingleProcessorAdmin::Instance().processor((*cursor)->model()).model();
    os << (*cursor)->asString() << "@" << m.asString() << ",";
  }//for

  os << std::endl;
  //os << "\n    Port End\n"; 

  return *this;
}//print()

/*******************************************************************
* Function Name: flattenInfluences
* For each port@Coupled in influenceList of addToPort, 
* replace it with port@Atomic
* Note: There are may be more than one port@Atomic associated with
*       port@Coupled
* Note: This is a recursive function
********************************************************************/
const Port &Port::flattenInfluences( Port &addToPort) const
{
  //must include "mainsimu.h" for the line below
  //MASSERT(MainSimulator::Instance().isFlatDEVS());

  InfluenceList::const_iterator cursor; //cursor is a pointer pointing to Port*
  for( cursor = influenceList.begin(); cursor != influenceList.end(); cursor++)
  {
    Model &m = SingleProcessorAdmin::Instance().processor((*cursor)->model()).model();
    //if (&m == NULL) continue;

    if ((m.mType() == Model::atomicType) ||
        (m.description() == Root::Instance().top().description()))
    {
      Model &mdl = SingleProcessorAdmin::Instance().processor(addToPort.model()).model();
      MainSimulator::Instance().AddLink(mdl.description(), addToPort.name(), m.description(), (*cursor)->name() );
      //Base condition: We reached a port@Atomic and it is not in addToPort's
      //                influenceList, so we add it to the list
      //cerr << "Bottom: "<< (*cursor)->asString() << "@" << m.asString()<<endl;
      if (!addToPort.findInfluence(*cursor)){
        //cerr << "ADD: " << (*cursor)->asString() << "@" << m.asString()<<endl;
        addToPort.addInfluence(**cursor);
	//addToPort.print(std::cout, 2);
      }//if
    }else if (m.mType() == Model::coupledType) {
      //cerr<< "Middle: " << (*cursor)->asString() << "@" << m.asString()<<endl;
      (*cursor)->flattenInfluences(addToPort);		//Recursion
      if ((*this == addToPort) && 
          (m.description() != Root::Instance().top().description()))
      {
        //cerr << "DEL: " << (*cursor)->asString() << "@" << m.asString()<<endl;
        InfluenceList::const_iterator next = cursor; 
	next++;
        addToPort.delInfluence(**cursor);
        Model &mdl = SingleProcessorAdmin::Instance().processor(addToPort.model()).model();
        MainSimulator::Instance().RemoveLink(mdl.description(), addToPort.name(), m.description(), (*cursor)->name());
	if (next == influenceList.end()){
	  break;	//-----|
        }else{
	  cursor = next;
        }//if-else
	//addToPort.print(std::cout, 2);
      }//if
    }//if-else
  }//for
  
  return *this;
}//Port::flattenInfluences()
