/*******************************************************************
*
*  DESCRIPTION: class RevAtomic
*
* 
*
*******************************************************************/

/** include files **/
#include "revatomic.h"             // base header
#include "message.h"		//class InternalMessage ExternalMessage
#include "modeladm.h"
#include "coupled.h"
#include "process.h"
#include "procadm.h"
#include "portlist.h"
#include "atomic.h"
#include "mainsimu.h"
#include "ini.h"
/*******************************************************************
* Function Name: holdIn
* Description: Changes the atomic state and the time left to the next change 
********************************************************************/
Model &RevAtomic::holdIn( const State &s, const Time &t )
{
	state( s );
	nextChange( t );
	return *this;
}

/*******************************************************************
* Function Name: passivate
* Description: Sets thes passive state and the next change to infinity 
********************************************************************/
Model &RevAtomic::passivate()
{
	state( passive );
	nextChange( Time::Inf );
	return *this;
}

bool RevAtomic::FullAddModel (std::string parentName, std::string modelName)
{
  //std::cout<<"RevAtomic fulladdmodel"<<std::endl;
  SingleModelAdm::Instance().newAtomic(modelName, modelName);
  Processor &sim_proc =SingleProcessorAdmin::Instance().processor(modelName);
  Model &mdl = sim_proc.model();
  Processor &proc = SingleProcessorAdmin::Instance().processor(parentName);
  Coupled &coupled(static_cast<Coupled &>(proc.model()));
  coupled.addModel(mdl);
  // coupled.addModelId(mdl.id());
  return true;

}

bool RevAtomic::AddModel(std::string parentName, std:: string modelName)
{
  std::cout<<"RevAtomic addmodel"<<std::endl;
  ProcessorAdmin::ProcessorDB::const_iterator it = SingleProcessorAdmin::Instance().getprocDB().begin();
  for(; it != SingleProcessorAdmin::Instance().getprocDB().end(); it++)
   std::cout<<"the processor is "<<(*it).first<<" "<<(*it).second<<std::endl;
  Processor &sim_proc =SingleProcessorAdmin::Instance().processor(modelName);
  Model &mdl = sim_proc.model();
  Processor &proc = SingleProcessorAdmin::Instance().processor(parentName);
  Coupled &coupled(static_cast<Coupled &>(proc.model()));
  coupled.addModel(mdl);

  return true;
}

bool RevAtomic::FullDelModel(std::string modelName)
{
 std::cout<<"(RevAtomic DelModel Function)"<<modelName<<std::endl;
  bool ret = true;
  Processor &proc = SingleProcessorAdmin::Instance().processor(modelName);
  ProcId pid = proc.id();
  std::cout<<"model id is "<<pid<<std::endl;
  Model &mdl = proc.model();  
  Coupled &coupled(static_cast<Coupled &>(mdl.parent()));
  std::cout<<"parent name is "<<mdl.parent().asString()<<std::endl;
  coupled.delModelId(mdl.id());
  if(mdl.mType() == Model::atomicType) 
    {
     std::cout<<"the model is an atomic model, name is "<<mdl.asString()<<std::endl;
   	  ProcessorAdmin::ProcessorDB::iterator pit =SingleProcessorAdmin::Instance().getprocDB().find(pid);
	  if(pit != SingleProcessorAdmin::Instance().getprocDB().end()) 
             {
               delete pit->second;
	       SingleProcessorAdmin::Instance().getprocDB().erase(pit);
               std::cout<<"delete processor id "<<pid<<std::endl;
             }
		
     }
 return ret;
}
bool RevAtomic::DelModel(std::string modelName)
{
 std::cout<<"(RevAtomic DelModel Function)"<<modelName<<std::endl;
  bool ret = true;
  Processor &proc = SingleProcessorAdmin::Instance().processor(modelName);
  ProcId pid = proc.id();
  std::cout<<"model id is "<<pid<<std::endl;
  Model &mdl = proc.model();  
  Coupled &coupled(static_cast<Coupled &>(mdl.parent()));
  std::cout<<"parent name is "<<mdl.parent().asString()<<std::endl;
  coupled.delModelId(mdl.id());

 return ret;

}

bool RevAtomic::DelLink(LINK link)
{
  std::cout<<"Del link function"<<std::endl;
  bool ret = true;
  Model * source = FindModel(link.sourceName);
  std::cout<<"source model is "<<source->asString()<<std::endl;
  Model * dest = FindModel(link.destName);
  std::cout<<"dest model is "<<dest->asString()<<std::endl;
  if ((source == NULL) || (dest == NULL)) ret = false;
 
  Port * sourceport;
  Port * destport;
  if(source->description() == this->parent().description())  sourceport = FindinputPort(link.sourceName, link.sourcePort);
  else  sourceport = FindoutputPort(link.sourceName, link.sourcePort);
     std::cout<<"source port is "<<sourceport->name()<<std::endl;
   if(dest->description() == this->parent().description())  destport = FindoutputPort(link.destName, link.destPort);
   else  destport = FindinputPort(link.destName, link.destPort);
      std::cout<<"dest port is "<<destport->name()<<std::endl;
      if((sourceport == NULL) || (destport == NULL)) ret = false; 
      else
      {
        std::cout<<"delete the influence of the source port"<<std::endl;
	InfluenceList::iterator pcursor = sourceport->influences().begin();
	for(; pcursor != sourceport->influences().end(); pcursor++)
	 std::cout<<"(before)the port name is "<<(*pcursor)->name()<<std::endl;
	pcursor = sourceport->influences().begin();
        while(pcursor != sourceport->influences().end())
	  {
	    std::string str = (*pcursor)->name();
	    std::cout<<"dest port name is "<<str<<std::endl;
            std::cout<<"dest port name is "<<link.destPort<<std::endl;
            if(str.compare(link.destPort) == 0) 
	      {
               std::cout<<"found the destPort "<<std::endl;

		sourceport->influences().erase(pcursor);
		//	destport->sublinkcount();
	
		break;
	      }
	    else
	      pcursor++;
	  }
  
      }
    
  return ret;
}


bool RevAtomic::AddLink(LINK link)
{
  std::cout<<"RevAtomic AddLink"<<std::endl;
  bool ret = true;
  Model * source = FindModel(link.sourceName);
  Model * dest = FindModel(link.destName);
  if((source == NULL) || (dest == NULL)) ret = false;

  Port * sourceport;
  Port * destport;
  if(source->description() == this->parent().description())
   {
     sourceport = FindinputPort(link.sourceName, link.sourcePort);
     //  if(sourceport == NULL) sourceport = &(source->addInputPort(link.sourceName));
   }
  else  
  {
    sourceport = FindoutputPort(link.sourceName, link.sourcePort);
    //  if(sourceport == NULL) sourceport = &(source->addOutputPort(link.sourceName));
  }
  if(dest->description() == this->parent().description()) 
  {
    destport = FindoutputPort(link.destName, link.destPort);
    //   if(destport == NULL) destport =  &(dest->addOutputPort(link.destName));
  }
  else  
  {
    destport = FindinputPort(link.destName, link.destPort);
    //  if(destport == NULL) destport =  &(dest->addInputPort(link.destName));
  }
  std::cout<<"source port is "<<sourceport<<" , dest port is "<<destport<<std::endl;
  if((sourceport == NULL) || (destport == NULL)) ret = false;
  else{
	  InfluenceList::iterator cursor;
	  for(cursor = sourceport->influences().begin(); cursor != sourceport->influences().end() && (*cursor)->model() < destport->model();
	      cursor++);
	  sourceport->influences().insert(cursor, destport);

  }

  return ret;
}

Model * RevAtomic::FindModel(std::string modelName)
{
  Processor &proc = SingleProcessorAdmin::Instance().processor(modelName);
  return &proc.model();
}

Port * RevAtomic::FindoutputPort(std::string ModelName, std::string PortName)
{
  std::cout<<"find output port function"<<std::endl;
  Port * pPort = NULL;
  Model * pmdl = FindModel(ModelName);
  std::cout<<"the model is "<<pmdl<<std::endl;
 
  PortList::iterator pit = pmdl->outputPorts().begin();
  while (pit != pmdl->outputPorts().end())
	{
	  if ((pit->second)->name().compare(PortName) == 0) 
	    {
	      pPort = pit->second;
	      break;
	    }
	  pit++;
	}
  std::cout<<"the port is "<<pPort<<std::endl;
 
  return pPort;
}

Port * RevAtomic::FindinputPort(std::string ModelName, std::string PortName)
{
  std::cout<<"finding input port "<<ModelName<<" "<<PortName<<std::endl;
  Port * pPort = NULL;
  Model * pmdl = FindModel(ModelName);

     PortList::iterator pit = pmdl->inputPorts().begin();
     while (pit != pmdl->inputPorts().end())
     {
	  if ((pit->second)->name().compare(PortName) == 0) 
	    {
	      pPort = pit->second;
	      break;
	    }
	  pit++;
     }

     
  return pPort;
  
}


Model &RevAtomic::GetNameset(std::string mName, int scomm)
{
  mNameset.clear();
  std::cout<<"(RevAtomic)struc change with scomm  "<<scomm<<"parent name is "<<mName<<std::endl;
  Ini::StGroupList::const_iterator it = MainSimulator::Instance().iniFile().stgroupList().begin();
  while( it != MainSimulator::Instance().iniFile().stgroupList().end())
    {
      unsigned int found;
      found = (it->first).find(mName, 0);
    
      if(found == 0)
	{
	  Ini::StDefList::const_iterator dit = it->second.find("scomm");
   
	  std::string str = (dit->second).front();


	  Ini::StrucMap::const_iterator scursor = MainSimulator::Instance().iniFile().getmap().find(mName);
          if(scursor != MainSimulator::Instance().iniFile().getmap().end())
	      {
		
		Ini::CommList::const_iterator ccursor = scursor->second.find(str);
		if(ccursor != scursor->second.end())
		  {
		    
	  int sc2int = ccursor->second;
	  if(sc2int == scomm)
	    {
	     std::cout<<"the pair is "<<mName<<" "<<str<<std::endl;
	      Ini::StIdList stlist( MainSimulator::Instance().iniFile().stdefinition(mName, str, "components"));
	      Ini::StIdList::const_iterator cursor;
	      for(cursor = stlist.begin(); cursor != stlist.end(); cursor++)
		{
		  std::string amName, atomicType;
		  std::cout<<"the current cursor is "<<(*cursor)<<std::endl;
		  bool isAtomic = MainSimulator::Instance().splitString(*cursor, amName, atomicType,'@');
		  if(isAtomic)
		    {
		      std::cout<<"the atomic model is "<<amName<<std::endl;
		      mNameset.insert(amName); 
		    }
		}//for
            }//if(sc2int == scomm)
		  }//if ccursor
	      }//if scursor
	     
      std::cout<<"the element is "<<it->first<<std::endl;  
	}//if found
       it++;
    }// while
 std::cout<<"the current name set is:"<<std::endl;
  ModelNameSet::iterator model_cur = mNameset.begin();
  for(; model_cur != mNameset.end(); model_cur++)
   std::cout<<(*model_cur)<<std::endl;
 
 std::cout<<"end of get component list"<<std::endl;
  
  return *this;
}

Model &RevAtomic::GetLinkset(std::string mName, int scomm) // we only get the full links inside the coupled model.
{
  linklist.clear();
  std::cout<<"(RevAtomic)get link list with scomm is "<<scomm<<std::endl;
  Ini::StGroupList::const_iterator it = MainSimulator::Instance().iniFile().stgroupList().begin();
  while(it != MainSimulator::Instance().iniFile().stgroupList().end())
    {
      unsigned int found;
      found = (it->first).find(mName, 0);
      if(found == 0)
	{
	  Ini::StDefList::const_iterator dit = it->second.find("scomm");
	  std::string str = (dit->second).front();
	  Ini::StrucMap::const_iterator scursor = MainSimulator::Instance().iniFile().getmap().find(mName);
	  if(scursor != MainSimulator::Instance().iniFile().getmap().end())
	    {
	      Ini::CommList::const_iterator ccursor = scursor->second.find(str);
	      int sc2int = ccursor->second;
	      if(sc2int == scomm)
		{
		  Ini::StIdList stlist(MainSimulator::Instance().iniFile().stdefinition(mName, str, "link"));
		  Ini::StIdList::const_iterator cursor = stlist.begin();
		  
		  while( cursor != stlist.end())
		    {
		      
		      std::string sourceName, sourcePort, destName, destPort;
                     
		      if(! MainSimulator::Instance().splitString(*cursor, sourcePort, sourceName, '@'))
			sourceName = this->parent().description(); 
		      
		      if(! MainSimulator::Instance().splitString(*(++cursor), destPort, destName, '@'))
                        destName = this->parent().description();
		     
                     	  std::cout<<"the link is "<<sourceName<<" "<<sourcePort<<" "<<destName<<" "<<destPort<<std::endl;
			  LINK link;
			  link.sourceName = sourceName;
			  link.sourcePort = sourcePort;
			  link.destName = destName;
			  link.destPort = destPort;
			  linklist.push_back(link);
			
		    
		      cursor++;
		     		      
		    }//while
		}//if sc2int
	    }//if scursor
	}// if found
      it++;
      std::cout<<"end of get link function"<<std::endl;
    }// while
  

  return *this;
}

Model &RevAtomic::diffmodel(ModelNameSet &nameset1, ModelNameSet &nameset2, ModelNameSet &diffname)
{
  // std::cout<<"diffmodel function"<<std::endl;
  //std::cout<<"nameset1 size is "<<nameset1.size()<<std::endl;
  //std::cout<<"nameset2 size is "<<nameset2.size()<<std::endl;
  ModelNameSet::iterator it, it1;
 
  it = nameset1.begin();
  while(it != nameset1.end())
    {
       it1 = nameset2.find(*it);
       if (it1 == nameset2.end()) diffname.insert(*it);
       it++;
    }
   std::cout<<"nameset1 size is "<<nameset1.size()<<std::endl;
   return *this;
}

Model &RevAtomic::difflink(LinkList &linkset1, LinkList &linkset2, LinkList &diff)
{
  // std::cout<<"(RevAtomic difflink)"<<std::endl;
 
  LinkList::iterator it, it1;
  int flag = 0;;
  it = linkset1.begin();
    while( it != linkset1.end())
    {
  
      for(it1 = linkset2.begin(); it1 != linkset2.end(); it1++)
	{
	 
	  if(equal((*it), (*it1))) 
              {
		//	std::cout<<"the link is found!"<<std::endl;
		flag = 1;
		break;
		}
	} 
      if(flag == 0 &&(it1 == linkset2.end())) {diff.push_back(*it); std::cout<<"the link is not found!"<<std::endl;}
      it++;
      flag = 0;
      it1 = linkset2.begin();
     
    }
    //for(it = diff.begin(); it != diff.end(); it++)
      //  std::cout<<"the link is  "<<(*it).sourceName<<" "<<(*it).sourcePort<<" "<<(*it).destName<<" "<<(*it).destPort<<std::endl;
  
  return *this;
}


bool RevAtomic::equal(LINK &link1, LINK &link2)
{
  // std::cout<<"(RevAtomic equal)"<<std::endl;
   if((link1.sourceName.compare(link2.sourceName) == 0)
      && (link1.sourcePort.compare(link2.sourcePort) == 0)
      && (link1.destName.compare(link2.destName) == 0)
      && (link1.destPort.compare(link2.destPort) == 0))
    return true;
  else return false;
}

Model &RevAtomic::setnameset(std::string mName)
{
  mNameset.insert(mName);
  return *this;
}

Model &RevAtomic::GetPortIn(std::string mName, int scomm)
{
  portinset.clear();
  std::cout<<"(RevAtomic GetPortIn)struc change with scomm  "<<scomm<<"parent name is "<<mName<<std::endl;
  Ini::StGroupList::const_iterator it = MainSimulator::Instance().iniFile().stgroupList().begin();
  while( it != MainSimulator::Instance().iniFile().stgroupList().end())
    {
      unsigned int found;
      found = (it->first).find(mName, 0);
    
      if(found == 0)
	{
	  Ini::StDefList::const_iterator dit = it->second.find("scomm");
   
	  std::string str = (dit->second).front();


	  Ini::StrucMap::const_iterator scursor = MainSimulator::Instance().iniFile().getmap().find(mName);
          if(scursor != MainSimulator::Instance().iniFile().getmap().end())
	      {
	
		Ini::CommList::const_iterator ccursor = scursor->second.find(str);
		if(ccursor != scursor->second.end())
		  {
		   
	  int sc2int = ccursor->second;
	  if(sc2int == scomm)
	    {
	     std::cout<<"the pair is "<<mName<<" "<<str<<std::endl;
	      Ini::StIdList stlist( MainSimulator::Instance().iniFile().stdefinition(mName, str, "in"));
	      Ini::StIdList::const_iterator cursor;
	      for(cursor = stlist.begin(); cursor != stlist.end(); cursor++)
		{
		  std::cout<<"cursor is "<<(*cursor)<<std::endl;
		  portinset.insert(*cursor);
		}
	    }
		  }
	      }
	}
      it++;
    }
  return *this;
}

Model &RevAtomic::GetPortOut(std::string mName, int scomm)
{
  portoutset.clear();
  std::cout<<"(RevAtomic GetPortOut)struc change with scomm  "<<scomm<<"parent name is "<<mName<<std::endl;
  Ini::StGroupList::const_iterator it = MainSimulator::Instance().iniFile().stgroupList().begin();
  while( it != MainSimulator::Instance().iniFile().stgroupList().end())
    {
      unsigned int found;
      found = (it->first).find(mName, 0);
    
      if(found == 0)
	{
	  Ini::StDefList::const_iterator dit = it->second.find("scomm");
   
	  std::string str = (dit->second).front();


	  Ini::StrucMap::const_iterator scursor = MainSimulator::Instance().iniFile().getmap().find(mName);
          if(scursor != MainSimulator::Instance().iniFile().getmap().end())
	      {
	
		Ini::CommList::const_iterator ccursor = scursor->second.find(str);
		if(ccursor != scursor->second.end())
		  {
		   
	  int sc2int = ccursor->second;
	  if(sc2int == scomm)
	    {
	      std::cout<<"the pair is "<<mName<<" "<<str<<std::endl;
	      Ini::StIdList stlist( MainSimulator::Instance().iniFile().stdefinition(mName, str, "out"));
	      Ini::StIdList::const_iterator cursor;
	      for(cursor = stlist.begin(); cursor != stlist.end(); cursor++)
		{
		  portoutset.insert(*cursor);
		}
	    }
		  }
	      }
	}
      it++;
    }
  return *this;
}

Model &RevAtomic::diffinport(PortNameSet &portin1, PortNameSet &portin2, PortNameSet &diffinport)
{
  PortNameSet::iterator it, it1;
 
  it = portin1.begin();
  while(it != portin1.end())
    {
       it1 = portin2.find(*it);
       if (it1 == portin2.end()) diffinport.insert(*it);
       it++;
    }
  return *this;
}

Model &RevAtomic::diffoutport(PortNameSet &portout1, PortNameSet &portout2, PortNameSet &diffoutport)
{
  PortNameSet::iterator it, it1;
 
  it = portout1.begin();
  while(it != portout1.end())
    {
       it1 = portout2.find(*it);
       if (it1 == portout2.end()) diffoutport.insert(*it);
       it++;
    }
  return *this;
}

bool RevAtomic::DelInputPort(std::string modelName, std::string InputPortName)
{
  bool ret = true;
  Processor &proc = SingleProcessorAdmin::Instance().processor(modelName);
  Coupled &coupled( static_cast<Coupled &>( proc.model() ) );
  if(proc.model().mType() != Model::coupledType) ret = false;
  if(FindinputPort(modelName, InputPortName) == NULL) ret = false;
  else{
    coupled.delInputPort(InputPortName);
  }

  return ret;
}

bool RevAtomic::DelOutputPort(std::string modelName, std::string OutputPortName)
{
  bool ret = true;
  Processor &proc = SingleProcessorAdmin::Instance().processor(modelName);
  Coupled &coupled( static_cast<Coupled &>( proc.model() ) );
  if(proc.model().mType() != Model::coupledType) ret = false;
  if(FindoutputPort(modelName, OutputPortName) == NULL) ret = false;
  else{
    coupled.delOutputPort(OutputPortName);
  }

  return ret;
}

bool RevAtomic::AddInputPort(std::string modelName, std::string InputPortName)
{
  std::cout<<"Add Input port, model name is "<<modelName<<" , Input port name is "<<InputPortName<<std::endl;
 bool ret = true;

   Processor &proc = SingleProcessorAdmin::Instance().processor(modelName);
  Coupled &coupled( static_cast<Coupled &>( proc.model() ) );
  if(proc.model().mType() != Model::coupledType) ret = false;
  if(FindinputPort(modelName, InputPortName) != NULL) 
   {
     ret = false;
     std::cout<<"the input port exists"<<std::endl;
   }
  else{
    std::cout<<"Call addInputPort !!"<<std::endl;
    coupled.addInputPort(InputPortName);
  }
  return ret;

}

bool RevAtomic::AddOutputPort(std::string modelName, std::string OutputPortName){
  bool ret = true;
  Processor &proc = SingleProcessorAdmin::Instance().processor(modelName);
  Coupled &coupled( static_cast<Coupled &>( proc.model() ) );
  if(proc.model().mType() != Model::coupledType) ret = false;
  if(FindoutputPort(modelName, OutputPortName) != NULL) 
   {
     ret = false;
     std::cout<<"the output port exists"<<std::endl;
   }
  else{
    coupled.addOutputPort(OutputPortName);
  }
  return ret;
}
