#ifndef _REVATOMIC_H
#define _REVATOMIC_H

/** include files **/
#include "model.h"
#include "atomic.h"
#include "message.h"
#include <set>
#include <list>
#include "port.h"

/**forward declarations**/
class MainSimulator ;
class RevSimulator ;
class StrucChangeMessage ;
class Coupled;



#define REVATOMIC_NAME "RevAtomic"

typedef struct 
{
  std::string sourceName;
  std::string sourcePort;
  std::string destName;
  std::string destPort;
}LINK;

class RevAtomic : public Model
{
 public:
  virtual ~RevAtomic(){} ; //destructor
  Model::ModelType mType() {return Model::revatomicType;}
  typedef std::set< std::string > ModelNameSet;
  typedef std::list< LINK > LinkList;

  typedef std::set< std::string > PortNameSet; 
 protected:
  friend class RevSimulator;
  friend class ModelAdmin;
  friend class FlatDEVSCoordinator;
  int oldscomm, newscomm;
enum State
  { 
    active,
    passive
  };
 RevAtomic(const std::string &name = "RevAtomic")
   :Model(name),
   st(passive)
   {}
 RevAtomic(const RevAtomic &); //copy constructor

 RevAtomic &operator = (const RevAtomic &); // Assignment operatior

 int operator == (const RevAtomic &) const ; //Equality operator
 virtual Model &InitFunction() = 0;
 virtual Model &internalFunction(const InternalMessage &) = 0;
 virtual Model &StartFunction() = 0;
 Model &holdIn(const State &, const Time &);

 Model &passivate();

 Model &setnameset(std::string mName);

 Model &state(const State &s)
   {st = s; return *this;}

 virtual std::string className() const
   {return  REVATOMIC_NAME;}

 bool AddModel (std::string parentName,  std::string modelName);
 bool FullAddModel(std::string parentName, std::string modelName);
 

 bool DelModel (std::string modelName);
 bool FullDelModel(std::string modelName);
 

 bool AddLink (LINK link);
 Model * FindModel(std::string modelName);
 
 bool DelLink(LINK link);
 

 bool DelInputPort(std::string modelName, std::string InputPortName);
 bool DelOutputPort(std::string modelName, std::string OutputPortName);
 bool AddInputPort(std::string modelName, std::string InputPortName);
 bool AddOutputPort(std::string modelName, std::string OutputPortName);


 Port * FindoutputPort(std::string ModelName, std::string PortName);
 Port * FindinputPort(std::string ModelName, std::string PortName);
 bool Clear();
 Model &diffmodel(ModelNameSet &nameset1, ModelNameSet &nameset2, ModelNameSet &diffname);
 Model &difflink(LinkList &linkset1, LinkList &linkset2, LinkList &diff);
 Model &diffinport(PortNameSet &portin1, PortNameSet &portin2, PortNameSet &diffinport);
 Model &diffoutport(PortNameSet &portout1, PortNameSet &portout2, PortNameSet &diffoutport);

 // private:
 friend class MainSimulator;
 enum State st;
 ModelNameSet mNameset;
 LinkList linklist;
 PortNameSet portinset;
 PortNameSet portoutset;
 
 Model &GetNameset(std::string mName, int scomm);
 Model &GetLinkset(std::string mName, int scomm);
 Model &GetPortIn(std::string mName, int scomm);
 Model &GetPortOut(std::string mName, int scomm);
 bool equal(LINK &link1, LINK &link2);
 
};
#endif //_REVATOMIC_H
