/*******************************************************************
*
*  DESCRIPTION: class Simulator
*
*  AUTHOR: Amir Barylko & Jorge Beyoglonian 
*
*  EMAIL: mailto://amir@dc.uba.ar
*         mailto://jbeyoglo@dc.uba.ar
*
*  DATE: 27/6/1998
*  
*******************************************************************/

/** include files **/
#include "simulat.h"          // header
#include "atomic.h"           // class Atomic
#include "message.h"          // class InitMessage
#include "msgadm.h"           // SingleMsgAdmin

/** public functions **/
/*******************************************************************
* Function Name: Simulator
* Description: constructor
********************************************************************/
Simulator::Simulator( Atomic *atomic ) 
: Processor( atomic )
  ,struc(0)
{}

/*******************************************************************
* Function Name: receive
* Description: set the time for the first internal event
********************************************************************/
Processor &Simulator::receive( const InitMessage &msg ) 
{
  Atomic &at( static_cast<Atomic &>( model() ) ) ;

  nextChange( Time::Inf ) ;

  at.initFunction() ;
  struc_rec = 0;

  DoneMessage done( msg.time(), Processor::id(), nextChange() ) ;

  FlatDebug().Active()?SingleMsgAdm::Instance().send( done, msg.procId() ):
                       SingleMsgAdm::Instance().send( done, at.parentId() );

  lastChange( msg.time() ) ;

  return *this ;
} //receive CollectMessage

/*******************************************************************
* Function Name: receive @ message
* Description: 
********************************************************************/
Processor &Simulator::receive( const CollectMessage &msg )
{
  //CollectMessage does not have to change
  //next change or last change!!!

  Atomic &at( static_cast<Atomic &>( model() ) ) ;
  MASSERT( at.absoluteNext() == msg.time() );
  InternalMessage m(msg.time(), msg.procId());
  at.outputFunction( m );  //calls model::sendOutput(), which then calls
                           //Processor::sendOutput(), which sends Y msg
                           //to at's parent
 
  //DoneMessage done( msg.time(), Processor::id(), Time::Zero); 
  DoneMessage done( msg.time(), Processor::id(), Time::Inf); 
  FlatDebug().Active()?SingleMsgAdm::Instance().send( done, msg.procId() ):
                       SingleMsgAdm::Instance().send( done, at.parentId() );

  return *this ;
}//receive CollectMessage

/*******************************************************************
* Function Name: receive q message
* Description: call the externalFunction and send done message
********************************************************************/
Processor &Simulator::receive( const ExternalMessage &msg ) 
{
  //Atomic &at( static_cast<Atomic &>( model() ) );
  //MASSERT( at.absoluteNext() >= msg.time() ) ;
  //nextChange( at.absoluteNext() - msg.time() ) ;
  //lastChange( msg.time() ) ;

  MASSERT( absoluteNext() >= msg.time() ) ;
  externalMsgs_m.add (static_cast<ExternalMessage *>(msg.clone()));

  //DoneMessage done( msg.time(), this->Processor::id(), nextChange() ) ;
  //SingleMsgAdm::Instance().send( done, at.parentId() ) ;

  return *this ;
}

/*******************************************************************
* Function Name: receive * Message
* Description: call to the OutputFunction and InternalFunction
********************************************************************/
Processor &Simulator::receive( const InternalMessage &msg ) 
{
  MASSERT( (msg.time() >= lastChange()) && (msg.time() <= absoluteNext()) );
  int value = (int)msg.value();
  //std::cout<<"(Simulator:)"<<std::endl;
  if(value == 0)
    {
  Atomic &at( static_cast<Atomic&>( model() ) ) ;
  
  if ( msg.time() < absoluteNext() )
  {
    nextChange(absoluteNext() - msg.time());
    at.externalFunction( externalMsgs_m );
    if(struc_rec == strucChange())
        externalMsgs_m.eraseAll();
  }
  else if ((msg.time() == absoluteNext()) && (externalMsgs_m.size() == 0))
  {
    nextChange( Time::Zero );
    at.internalFunction( msg );
  }
  else if ( msg.time() == absoluteNext() && externalMsgs_m.size() != 0 )
  {
    nextChange( Time::Zero );
    at.confluentFunction( msg, externalMsgs_m );
    if(struc_rec == strucChange())
      externalMsgs_m.eraseAll();
  }

  lastChange( msg.time() );

  if(struc_rec != strucChange())
    {
      std::cout<<"Simulator: receive a structural change"<<strucChange()<<std::endl;
  DoneMessage done( msg.time(), this->Processor::id(), nextChange(), strucChange() );
  struc_rec = strucChange();
  std::cout<<"###############################points(1) atomic mode name is "<<at.description()<<std::endl;
  std::cout<<"###############################points(1) atomic mode 's parent name is "<<at.parentId()<<std::endl;
  FlatDebug().Active()?SingleMsgAdm::Instance().send(done, msg.procId()):
                       SingleMsgAdm::Instance().send(done, at.parentId());
    } 
  else{
  DoneMessage done( msg.time(), this->Processor::id(), nextChange() );

  FlatDebug().Active()?SingleMsgAdm::Instance().send( done, msg.procId() ):
                       SingleMsgAdm::Instance().send( done, at.parentId() );
  }
    }

  return *this ;
}
/*******************************************************************
* Function Name: receive Start Message
* Description: call to the OutputFunction and InternalFunction
********************************************************************/
Processor &Simulator::receive( const StartMessage &msg ) 
{
  Atomic &at( static_cast<Atomic&>( model() ) ) ;
  nextChange( Time::Inf ) ;

  at.StartFunction(msg) ;
  lastChange( msg.time() );
  struc_rec = strucChange();
  DoneMessage done( msg.time(), this->Processor::id(), nextChange() );

  FlatDebug().Active()?SingleMsgAdm::Instance().send( done, msg.procId() ):
                       SingleMsgAdm::Instance().send( done, at.parentId() );

  return *this ;
}

/*******************************************************************
* Function Name: id
********************************************************************/
Processor &Simulator::id( const ProcId &pid )
{
  this->Processor::id( pid ) ;

  MASSERT( pmodel() ) ;

  pmodel()->id( pid ) ;

  return *this ;
}
