/*******************************************************************
*
*  DESCRIPTION: class StandAloneLoader
*
*  AUTHOR:    Amir Barylko & Jorge Beyoglonian 
*  Version 2: Daniel A. Rodriguez.
*  Version 3: Mohammad Moallemi :Realtimerun and emtyeventfile functions added
*
*  EMAIL: mailto://amir@dc.uba.ar
*         mailto://jbeyoglo@dc.uba.ar
*         mailto://drodrigu@dc.uba.ar
*         mailto://moallemi@sce.carleton.ca
*
*  DATE: 27/6/1998
*  DATE: 29/4/1999
*  DATE: 10/9/2008
*
*******************************************************************/

/** include files **/
#include <fstream>
#include <sstream>
#include <math.h>

#include "stdaload.h"      // base class
#include "cdTime.h"
#include "distri.h"
#include "impresion.h"
#include "realprec.h"

extern "C"
{
  #include "randlib.h"
}

#define  CTE_STR_W  "The format for the -w parameter is: -wXX-YY where XX and YY are integer numbers."
/*******************************************************************
 * this variables have to be declared here in order to enable the
 * prior to use it
 *******************************************************************/
//parallelPort pport2;
bool lpt2on = false;
/*******************************************************************/

int getWidth( const char *str )
{
  char  aux[100];
  int  i = 0;

  while (*str != 0 && *str != '-')
  {
    aux[i++] = *str;
    str++;
  }
  MASSERTMSG(*str == '-', CTE_STR_W );

  aux[i] = 0;
  return atoi(aux);
}

int getPrecision( const char *str )
{
  char  aux[100];
  int  i = 0;

  while (*str != 0 && *str != '-')
    str++;

  MASSERTMSG(*str == '-', CTE_STR_W );
  str++;

  while (*str != 0)
  {
    aux[i++] = *str;
    str++;
  }
  aux[i] = 0;
  return atoi(aux);
}

/*******************************************************************
* Function Name: StandAloneLoader
* Description: Constructor
********************************************************************/
StandAloneLoader::StandAloneLoader( int argc, const char *argv[] )
: iniName( "model.ma" )
, outName( "/dev/null" )
, logName( "/dev/null" )
, evName( "" )
, evalDebugName( "/dev/null" )
, flatDebugName( "/dev/null" )
, devsMdlDebugName( "/dev/null" )
, parserFileName( "stdout" )
{
  stopTime( Time::Inf ) ;
  endeventTime ( Time::Inf ) ;
  printParserInfo(false);
  preprocessor = 1;
  DebugCellRules().Active( false );
  UseQuantum().Active( false );
  isRealHardRun(false);
  EmptyEventFile(false);

  Impresion::Default.Width( 12 );
  Impresion::Default.Precision( 5 );

  //////////////////////////////////////////////////////////////////////
  // Set the seed for the random number generator with the clock's value
  std::string  str;
  char  c;
  long  seed1, seed2;
  
  time_t seconds;
  time (&seconds);
  srand(seconds);
  for (register int i = 1; i <= 20; i++)
  {
    c = 50 + static_cast<int>( 50 * rand() / (RAND_MAX + 1) );
    str += c;
  }
  phrtsd( const_cast< char *> ( str.c_str() ), &seed1, &seed2);
  setall(seed1, seed2);
  /////////////////////////////////////////////////////////////////////  

  while( --argc )
  {
    if( argv[ argc ][ 0 ] == '-' )
    {
      switch( argv[ argc ][ 1 ] )
      {
        case 'b': // bypass the preprocessor
          preprocessor = 0;
          break;

        case 'c': // print DEVS model mode
          devsMdlDebugName = argv[ argc ] + 2 ;
          break;

        case 'd': // Set the value of TOL
          RealPrecision::Tol.precision( atof( argv[argc] + 2 ) );
          break;

        case 'f': // evaluate debug mode
          flatDebugName = argv[ argc ] + 2 ;
	  	  isFlatDevs = true;
          break;
          
        case 'g': //v3 real hardware
		  isRealHardRun(true);
		  endeventTime( Time( argv[ argc ] + 2 ) ) ; //v3
          break;

        case 'e': // event file
          evName = argv[ argc ] + 2 ;
          break;

        case 'h': // help
          std::cout << argv[ 0 ] << " [-cbdefhlmopqrstvw]" << std::endl
          << "\tb: bypass the preprocessor (macros are ignored)" << std::endl
          << "\tc: print the DEVS model into a file" << std::endl
          << "\td: set tolerance used to compare real numbers" << std::endl
          << "\te: events file (default: none)" << std::endl
          << "\tf: flat coordinator mode" << std::endl
          << "\tg: get real! - run and simulate in real time" << std::endl
          << "\t   (requires a working LPT1 and root access)" << std::endl
          << "\th: show this help" << std::endl 
          << "\tl: message log file (default: /dev/null)" << std::endl
          << "\tm: model file (default : model.ma)" << std::endl
          << "\to: output (default: /dev/null)" << std::endl
          << "\tp: print extra info when the parsing occurs (only for cells models)" << std::endl
          << "\tq: use quantum to calculate cells values" << std::endl
          << "\tr: debug cell rules mode (only for cells models)" << std::endl
          << "\ts: show the virtual time when the simulation ends (on stderr)" << std::endl
          << "\tS: all clock time on log and output files" << std::endl
          << "\tt: stop time (default: Infinity)" << std::endl
          << "\tv: evaluate debug mode (only for cells models)" << std::endl
          << "\tw: sets the width and precision (with form xx-yy) to show numbers" << std::endl
          << "\tW: used Wall-clock time (Real-Time approach)" << std::endl;
          exit(0);
          break;

        case 'l': // log name
          logName = argv[ argc ] + 2 ;
          break;

        case 'm': // file de modelos a cargar
          iniName = argv[ argc ] + 2 ;
          break;

        case 'o': // output stream
          outName = argv[ argc ] + 2 ;
          break;

        case 'p': // Print extra parsing info
          printParserInfo(true);
          parserFileName = argv[ argc ] + 2;
          ParserDebug().Active( true );
          break;

        case 'r': // Debug Cells rules
          DebugCellRules().Active( true );
          break;

        case 'q': // Set the value of the Quantum
          UseQuantum().Value( atof( argv[argc] + 2 ) );
          UseQuantum().Active( true );
          break;

        case 's':  // Show virtual time when simulation ends
          ShowVirtualTimeWhenFinish().Active( true );
          break;

        case 'S':  // Show all clock time on log and output files
          showWallClockTime(true);
          break;

        case 't': // hora de finalizacion de la simulacion
          stopTime( Time( argv[ argc ] + 2 ) ) ;
          break;

        case 'v': // evaluate debug mode
          evalDebugName = argv[ argc ] + 2 ;
          break;

        case 'w': // set the width & precision to show real numbers
          Impresion::Default.Width( getWidth( argv[ argc ] + 2 ));
          Impresion::Default.Precision( getPrecision( argv[ argc ] + 2 ));
          break;

        case 'W': // use wall-clock time
          isRealTimeRun(true);
          break;

        default:
          std::cout << "Invalid parameter " << argv[ argc ] << "!" << std::endl ;
          exit(0);
      }
    } else
    {
      std::cout << "Invalid parameter " << argv[ argc ] << "!" << std::endl ;
      exit(0);
    }
  }
}

/*******************************************************************
* Function Name: ~StandAloneLoader
********************************************************************/
StandAloneLoader::~StandAloneLoader()
{
	// N/A
}

/** private functions **/

/*******************************************************************
* Function Name: loadData
********************************************************************/
SimLoader &StandAloneLoader::loadData()
{
  std::cout << "N-CD++: A Tool to Implement n-Dimensional Cell-DEVS models" << std::endl;
  std::cout << "----------------------------------------------------------" << std::endl;
  std::cout << "Version 2.0-R.45 December-1999" << std::endl;
  std::cout << "Daniel Rodriguez, Gabriel Wainer, Amir Barylko, Jorge Beyoglonian" << std::endl;
  std::cout << "Departamento de Computacion. Facultad de Ciencias Exactas y Naturales." << std::endl;
  std::cout << "Universidad de Buenos Aires. Argentina." << std::endl;
  std::cout << std::endl;

  std::cout << "Loading models from " << iniName << std::endl ;
  std::cout << "Loading events from " << evName << std::endl ;
  std::cout << "Message log: " << logName << std::endl ;
  std::cout << "Output to: " << outName << std::endl ;
  std::cout << "Tolerance set to: " <<  RealPrecision::Tol.precision() << std::endl;

  MASSERTMSG( RealPrecision::Tol.precision() >= 0, "The precision can't be negative.");

  std::cout << "Configuration to show real numbers: Width = " 
            << Impresion::Default.Width() 
            << " - Precision = " <<  Impresion::Default.Precision() 
            << std::endl;

  std::cout << "Quantum: ";
  if (UseQuantum().Active())
    std::cout <<  UseQuantum().Value() << std::endl;
  else
    std::cout << "Not used" << std::endl;

  std::cout << "Evaluate Debug Mode = ";
  if (evalDebugName == "/dev/null")
  {
    std::cout << "OFF" << std::endl;
    EvalDebug().Active( false );
  }
  else
  {
    std::cout << "ON\tOutput of evaluated data = " 
              << evalDebugName << std::endl;
    EvalDebug().Active( true );
  }

  std::cout << "DEVS Model Debug Mode = ";
  if (devsMdlDebugName == "/dev/null") {
    std::cout << "OFF" << std::endl;
    DevsMdlDebug().Active( false );
  } else {
    std::cout << "ON\tDEVS Model File  = " << devsMdlDebugName << std::endl;
    DevsMdlDebug().Active( true );
  }

  std::cout << "Flat Debug Mode = ";
  if (flatDebugName == "/dev/null") {
    std::cout << "OFF" << std::endl;
    FlatDebug().Active( false );
  } else {
    std::cout << "ON\tOutput of flat debug data = " 
              << flatDebugName << std::endl;
    FlatDebug().Active( true );
  }

  std::cout << "Debug Cell Rules Mode = " 
            << (DebugCellRules().Active() ? "ON":"OFF") << std::endl;

  if (!preprocessor)
  {
    std::cout << "Bypassing preprocessor." << std::endl;
    models = new std::fstream( iniName.c_str(), std::ios::in );
  }
  else
  {
    MacroExpansion(iniName);
    models = new std::fstream( MacroExpansion().expand().c_str(), std::ios::in )  ;
    std::cout << "Temporary File created by Preprocessor = " << MacroExpansion().tempFileName() << std::endl;
  }

  std::cout << "Printing parser information = ";
  if (printParserInfo())
  {
    std::cout << "ON" << "\tUsing file = " << parserFileName << std::endl;
  }
  else
  {
    std::cout << "OFF" << std::endl;
  }

  std::cout << "Real Hardware I/O (USB)= ";//v3
  if (isRealHardRun())
  {
    std::cout << "ON" << std::endl;
    if(evName == ""){
       std::cout << "Deadline for external events: "; //test
  	   if (endeventTime() == Time::Inf) std::cout << "Infinity.";
  	   else std::cout << endeventTime().asString() << std::endl ; //end test
  	   }
  }
  else
  {
    std::cout << "OFF" << std::endl;
  }

  std::cout << std::endl << "Starting simulation. Stop at time: ";
  if (stopTime() == Time::Inf)
    std::cout << "Infinity.";
  else
    std::cout << stopTime().asString();
  std::cout << std::endl;

  if( evName == "" ){
    events = new std::stringstream();
    EmptyEventFile(true); }
  else
    events = new std::fstream( evName.c_str(), std::ios::in );

  if( logName == "-" || logName == "" )
    log = &std::cout;
  else
    log = new std::fstream( logName.c_str(), std::ios::out );

  if( outName == "-" || outName == "" )
    output = &std::cout ;
  else
    output = new std::fstream( outName.c_str(), std::ios::out ) ;

  if( evalDebugName == "-" || evalDebugName == "" )
    EvalDebug().Stream( &std::cout );
  else
    EvalDebug().Stream( new std::fstream( evalDebugName.c_str(), std::ios::out ) );

  if( flatDebugName == "-" || flatDebugName == "" )
    FlatDebug().Stream( &std::cout );
  else
    FlatDebug().Stream( new std::fstream( flatDebugName.c_str(), std::ios::out ) );

  if( devsMdlDebugName == "-" || devsMdlDebugName == "" )
    DevsMdlDebug().Stream( &std::cout );
  else
    DevsMdlDebug().Stream(new std::fstream(devsMdlDebugName.c_str(), std::ios::out));

  if( parserFileName == "-" || parserFileName == "stdout" )
    ParserDebug().Stream( &std::cout );
  else
    ParserDebug().Stream( new std::fstream( parserFileName.c_str(), std::ios::out ));

  return *this;   
}

/*******************************************************************
* Function Name: writeResults
********************************************************************/
SimLoader &StandAloneLoader::writeResults()
{
  std::cout << "Simulation ended!" << std::endl;
  delete models ;  models = NULL;
  delete events ;  events = NULL; 

  if( logName != "-" )
  {
    delete log ;  
    log = NULL ;
  }

  if( outName != "-" )
  {
    delete output ;  
    output = NULL ;
  }

  if (preprocessor)
    delete &(MacroExpansion());

  return *this;   
}

/*******************************************************************
* Function Name: isRealTimeRun
********************************************************************/
const bool StandAloneLoader::isRealTimeRun() const
{
  return isrealtimerun_m;
}

/*******************************************************************
* Function Name: isRealTimeRun (bool)
********************************************************************/
SimLoader &StandAloneLoader::isRealTimeRun(bool bisisRealTimeRun)
{
  isrealtimerun_m = bisisRealTimeRun;
  return *this;
}

/*******************************************************************
* Function Name: isRealRun
********************************************************************/
const bool StandAloneLoader::isRealHardRun() const
{
  return realsimulation_m;
}

/*******************************************************************
* Function Name: isRealRun (bool)
********************************************************************/
SimLoader &StandAloneLoader::isRealHardRun(bool varisRealRun)
{
  realsimulation_m = varisRealRun;
  return *this;
}

/*******************************************************************
* Function Name: EmptyEventFile
********************************************************************/
const bool StandAloneLoader::EmptyEventFile() const
{
  return realevents_m;
}

/*******************************************************************
* Function Name: EmptyEventFile (bool)
********************************************************************/
SimLoader &StandAloneLoader::EmptyEventFile(bool varisEmptyEvFile)
{
  realevents_m = varisEmptyEvFile;
  return *this;
}

/*******************************************************************
* Function Name: showWallClockTime
********************************************************************/
const bool StandAloneLoader::showWallClockTime() const
{
  return showwallclocktime_m;
}

/*******************************************************************
* Function Name: showWallClockTime (bool)
********************************************************************/
SimLoader &StandAloneLoader::showWallClockTime(bool showWallClockTime)
{
  showwallclocktime_m = showWallClockTime;
  return *this;   
}
 
